// -*- C++ -*-

/* 
 * app.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include <algo.h>
#include <list>

#include "app.h"
#include "guppi.h"
#include "columnlist.h"
#include "plotedit.h"
#include "plotwindow.h"
#include "columnimporterfile.h"
#include "columnimporterstore.h"
#include "muncher.h"
#include "dialogtracker.h"
#include "apptracker.h"
#include "importer.h"
#include "scatterdoc.h"

Muncher<Importer<ColumnDoc> > columnimporter_muncher;

static void munch(Importer<ColumnDoc>* importer) 
{ 
  columnimporter_muncher.munch(importer); 
}

class GuppiAppImpl : public DialogTracker, 
		     public AppTracker,
		     public Gtk_Signal_Base {
public:
  GuppiAppImpl(GuppiDoc* doc,
	       const gchar* geometry,
	       GuppiApp & app);
  ~GuppiAppImpl();

  // These have to be public for the callbacks,
  //  but don't use them. 
  // If I ever get un-lazy the callbacks could be declared
  //  friend
  // Menu actions.
  void about();
  void new_doc();
  void open();
  void import_file(size_t i);
  void save_as();
  void preferences();
  void import_columns(Importer<ColumnDoc>* ci);
  void new_scatter();

  // abuse potential
  GuppiDoc* doc() { return doc_; }

protected:
  // setup/reset
  void init();
  void view_doc();
  void make_menus();
  void set_sensitivity();
  void set_doc(GuppiDoc * doc);
  
  void display_all_plots();

  // These are both callbacks and manually callable.
  void display_icon(const GuppiDoc & gd);
  void display_name(const GuppiDoc & gd);

  void add_plot(const PlotDoc & pd);
  void delete_plot(const PlotDoc & pd);
  
  void display_plot_icon(const PlotDoc & pd);
  void display_plot_name(const PlotDoc & pd);
  void display_plot(const PlotDoc & pd);

  // callbacks
  gint plots_button_press_cb  (GdkEventButton* be);

  void plot_edit();
  void plot_window();
  void plot_delete();

private:
  GuppiDoc * doc_;
  GuppiApp & app_;

  list<Gtk_MenuItem*> import_menuitems_;

  // GUI cruft.
  Gtk_Notebook notebook_;
  ColumnList columns_page_;
  Gtk_VBox plots_page_;
  Gtk_Label columns_label_;
  Gtk_Label plots_label_;
  Gnome_IconList plots_icons_;

  Gtk_Menu plots_popup_;
  Gtk_MenuItem plot_edit_;
  Gtk_MenuItem plot_window_;
  Gtk_MenuItem plot_delete_;

  // Last one clicked
  PlotDoc* clicked_pd_;
};


GuppiApp::GuppiApp(GuppiDoc* doc, const gchar* geometry) :
  Gnome_App(Guppi::APPNAME, ""), 
  impl_(new GuppiAppImpl(doc,geometry,*this))
{
  
}

GuppiApp::~GuppiApp()
{
  delete impl_;
#ifdef GNOME_ENABLE_DEBUG
  g_print("~GuppiApp\n");
#endif
}

void 
GuppiApp::self_munch()
{
  big_fish.munch(this);
}

GuppiAppImpl::GuppiAppImpl(GuppiDoc* doc,
			   const gchar * geometry,
			   GuppiApp & app) : 
  doc_(doc), app_(app), 
  notebook_(), columns_page_(*doc),
  plots_page_(FALSE,GNOME_PAD), columns_label_(_("Columns")), 
  plots_label_(_("Plots")), plots_icons_(),

  plots_popup_(),
  plot_edit_(_("Edit...")),
  plot_window_(_("Display")),
  plot_delete_(_("Delete")),
  clicked_pd_(0)
{
  plots_popup_.append(&plot_edit_);
  plot_edit_.show();
  plots_popup_.append(&plot_window_);
  plot_window_.show();
  plots_popup_.append(&plot_delete_);
  plot_delete_.show();

  connect_to_method(plot_edit_.activate,
		    this,
		    &GuppiAppImpl::plot_edit);

  connect_to_method(plot_window_.activate,
		    this,
		    &GuppiAppImpl::plot_window);

  connect_to_method(plot_delete_.activate,
		    this,
		    &GuppiAppImpl::plot_delete);

  // fixme parse geometry
  //  app_.set_usize(300,500);

  init();
}

GuppiAppImpl::~GuppiAppImpl()
{
  list<Gtk_MenuItem*>::iterator i = import_menuitems_.begin();
  while ( i != import_menuitems_.end() ) {
    delete *i;
    ++i;
  }
}

void
GuppiAppImpl::init()
{
  make_menus();

  app_.set_contents(&notebook_);  

  plots_icons_.set_policy(GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);

  connect_to_method(plots_icons_.button_press_event,
		    this,
		    &GuppiAppImpl::plots_button_press_cb);

  plots_page_.pack_start(&plots_icons_, TRUE, TRUE, GNOME_PAD);

  notebook_.append_page(&columns_page_,&columns_label_);
  notebook_.append_page(&plots_page_,&plots_label_);
  notebook_.show_all();

  view_doc();
}


// For initial setup, we have to slurp everything;
//  then up-to-dateness is preserved by signals.
void
GuppiAppImpl::view_doc()
{
  g_return_if_fail(doc_ != 0);

  display_name(*doc_);

  // Arrange to die if the doc dies.
  connect_to_method(doc_->destroy, &app_,
		    &GuppiApp::self_munch);

  connect_to_method(doc_->plot_added,
		    this,
		    &GuppiAppImpl::add_plot);

  connect_to_method(doc_->plot_deleted,
		    this,
		    &GuppiAppImpl::delete_plot);
  
  // fixme add other doc_ signals

  display_all_plots();

  set_sensitivity();
}

void
GuppiAppImpl::set_doc(GuppiDoc* doc)
{
  g_warning("GuppiAppImpl::set_doc(GuppiDoc* doc) not implemented");
}

void
GuppiAppImpl::display_all_plots()
{
  plots_icons_.freeze();
  GuppiDoc::plot_iterator j = doc_->plots_begin();
  while ( j != doc_->plots_end() ) {
    add_plot(**j);
    ++j;
  }
  plots_icons_.thaw();
}

//////////////// callbacks begin

void
GuppiAppImpl::display_icon(const GuppiDoc & gd)
{
  g_warning("GuppiAppImpl::display_icon not implemented");
}

void
GuppiAppImpl::display_name(const GuppiDoc & gd)
{
  app_.set_title((gd.get_name()).c_str());  
}

void
GuppiAppImpl::add_plot(const PlotDoc & pd)
{
  g_print("GuppiAppImpl::add_plot(const PlotDoc & pd)\n");
  connect_to_method(pd.name_changed, this, 
		    &GuppiAppImpl::display_plot_name);
  connect_to_method(pd.icon_changed, this, 
		    &GuppiAppImpl::display_plot_icon);
  display_plot(pd);
}

void
GuppiAppImpl::delete_plot(const PlotDoc & pd)
{
  plots_icons_.clear();
  display_all_plots();
}

void 
GuppiAppImpl::display_plot_icon(const PlotDoc & pd)
{
  display_plot(pd);
}

void 
GuppiAppImpl::display_plot_name(const PlotDoc & pd)
{
  display_plot(pd);
}

void 
GuppiAppImpl::display_plot(const PlotDoc & pd)
{
  int pos;

  g_print("GuppiAppImpl::display_plot(const PlotDoc & pd)\n");

  pos = plots_icons_.find_icon_from_data(&pd);

  if ( pos != -1 ) {
    // icon already exists for this PlotDoc - reinsert with new 
    //  characteristics.
    plots_icons_.freeze();
    plots_icons_.remove(pos);
    plots_icons_.insert(pos,pd.get_icon().c_str(),pd.get_name().c_str());
    plots_icons_.thaw();
  }
  else {
    // no icon yet, so add one.
    pos = plots_icons_.append(pd.get_icon().c_str(),pd.get_name().c_str());
  }
  
  plots_icons_.set_icon_data(pos,&pd);
}

//////////// end of doc callbacks.

gint 
GuppiAppImpl::plots_button_press_cb(GdkEventButton* be)
{
  gint pos = plots_icons_.get_icon_at(be->x,be->y);

  if (pos < 0) return FALSE; // no icon there.

  clicked_pd_ = (PlotDoc *)plots_icons_.get_icon_data(pos);

  if (clicked_pd_ != 0) {
    g_print("Button press, plot data %s\n", 
	    clicked_pd_->get_name().c_str());
    if (be->button != 3) {
      return FALSE;
    }
    else {
      g_print("Doing popup\n");
      plots_popup_.popup(0,0,0,0,be->button,be->time);
      return TRUE;
    }
  }
  else {
    g_warning("Null data on icon %d", pos);
    return FALSE;
  }
}

void
GuppiAppImpl::new_doc()
{
  big_fish.new_doc();
}

void 
GuppiAppImpl::open()
{
  g_print("Open\n");
}

void
GuppiAppImpl::import_file(size_t i)
{
  Importer<ColumnDoc>* ci = 
    global_column_importer_store.make_importer_number(i);
  connect_to_method(ci->finished, this,
		    &GuppiAppImpl::import_columns);
  connect_to_function(ci->cancelled,
		      munch);
}

void 
GuppiAppImpl::save_as()
{
  g_print("Save As\n");
}

void 
GuppiAppImpl::preferences()
{
  g_print("Prefs\n");
}

void 
GuppiAppImpl::import_columns(Importer<ColumnDoc>* ci)
{
  list<ColumnDoc*> * columns = ci->get_docs();

  if (columns != 0) {
    list<ColumnDoc*>::iterator i = columns->begin();
    while (i != columns->end()) {
      g_print("Adding imported column: %s\n", (*i)->get_name().c_str());
      doc_->add_column(**i);
      ++i;
    }
    delete columns;
    // note that doc_ now owns the columns and will delete them
    //  when it dies.
  }

  munch(ci);
}

void
GuppiAppImpl::new_scatter()
{
  ScatterDoc* sd = new ScatterDoc;

  doc_->add_plot(*sd);
}

void
GuppiAppImpl::plot_edit()
{
  g_return_if_fail(clicked_pd_ != 0);

  PlotEdit* pe = new PlotEdit(*clicked_pd_,*doc_);
  pe->show();  
}

void 
GuppiAppImpl::plot_window()
{
  g_return_if_fail(clicked_pd_ != 0);

  PlotWindow* pw = new PlotWindow(*clicked_pd_);
  pw->show();
}

void
GuppiAppImpl::plot_delete()
{
  g_return_if_fail(clicked_pd_ != 0);

  doc_->delete_plot(clicked_pd_);
}


///////////////////////////////// Menu cruft

// Bunch of callbacks - they just call the relevant methods on the 
//  object sent in as data.

static void 
about_cb(GtkWidget * w, gpointer data)
{
  g_return_if_fail(data != 0);
  big_fish.about();
}

static void
new_doc_cb(GtkWidget * w, gpointer data)
{
  g_return_if_fail(data != 0);
  g_print("new_doc_cb(GtkWidget * w, gpointer data)\n");
  ((GuppiAppImpl *)data)->new_doc();
}

static void
open_cb(GtkWidget * w, gpointer data)
{
  g_return_if_fail(data != 0);
  ((GuppiAppImpl *)data)->open();
}

static void
save_cb(GtkWidget * w, gpointer data)
{
  g_return_if_fail(data != 0);
  big_fish.save_doc( ((GuppiAppImpl*)data)->doc() );
}

static void
save_as_cb(GtkWidget * w, gpointer data)
{
  g_return_if_fail(data != 0);
  ((GuppiAppImpl *)data)->save_as();
}

static void
close_cb(GtkWidget * w, gpointer data)
{
  g_return_if_fail(data != 0);
  big_fish.close_doc(((GuppiAppImpl *)data)->doc());
}

static void 
exit_cb(GtkWidget * w, gpointer data)
{
  big_fish.exit();
}

static void 
new_scatter_cb(GtkWidget * w, gpointer data)
{
  g_return_if_fail(data != 0);
  ((GuppiAppImpl *)data)->new_scatter();
}

static void
preferences_cb(GtkWidget * w, gpointer data)
{
  g_return_if_fail(data != 0);
  ((GuppiAppImpl *)data)->preferences();
}


static GnomeUIInfo help_menu[] = {
  GNOMEUIINFO_HELP(Guppi::APPNAME),
  {
    GNOME_APP_UI_ITEM, N_("About..."), 
    N_("Tell about this application"), 
    about_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT, 0, 0, NULL 
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo file_menu[] = {
  {
    GNOME_APP_UI_ITEM, 
    N_("New Data Set"), N_("Open a new data set document"), 
    new_doc_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW, 'n', 
    GDK_CONTROL_MASK, NULL 
  },
  {
    GNOME_APP_UI_ITEM, 
    N_("Open..."), N_("Open a new file in this window"), 
    open_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_OPEN, 'o', 
    GDK_CONTROL_MASK, NULL 
  },
#define MENU_SAVE_POS 2
  {
    GNOME_APP_UI_ITEM, 
    N_("Save"), N_("Save file to disk"), 
    save_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE, 's', 
    GDK_CONTROL_MASK, NULL 
  },
#define MENU_SAVEAS_POS 3
  {
    GNOME_APP_UI_ITEM, 
    N_("Save As..."), N_("Save file to disk"), 
    save_as_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE, 'f', 
    GDK_CONTROL_MASK, NULL 
  },
  {
    GNOME_APP_UI_ITEM, N_("Close"), 
    N_("Close this window"),
    close_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXIT, 'c', 
    GDK_CONTROL_MASK, NULL 
  },
  GNOMEUIINFO_SEPARATOR,
  {
    GNOME_APP_UI_ITEM, N_("Exit"), 
    N_("Quit the application"),
    exit_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXIT, 'q', 
    GDK_CONTROL_MASK, NULL 
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo new_plot_menu[] = {
  { 
    GNOME_APP_UI_ITEM,
    N_("Scatter"), N_("Create a new scatter plot"), 
    new_scatter_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW, 0, 
    GDK_CONTROL_MASK, NULL 
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo import_column_menu[] = {
#if 0
  { 
    GNOME_APP_UI_ITEM,
    N_("from file..."), N_("Read columns from a file."), 
    import_file_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_OPEN, 0, 
    GDK_CONTROL_MASK, NULL 
  },
#endif
  GNOMEUIINFO_END
};

static GnomeUIInfo document_menu[] = {
  GNOMEUIINFO_SUBTREE( N_("New Plot"), new_plot_menu ),
#define MENU_IMPORT_POS 1
  GNOMEUIINFO_SUBTREE( N_("Import Column"), import_column_menu ),
  GNOMEUIINFO_END
};

static GnomeUIInfo prefs_menu[] = {
  {
    GNOME_APP_UI_ITEM, N_("Preferences..."), 
    N_("Change application preferences"),
    preferences_cb, NULL, NULL,
    GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PREF, 'p', 
    GDK_CONTROL_MASK, NULL 
  },
  GNOMEUIINFO_END
};

static GnomeUIInfo main_menu[] = {
  GNOMEUIINFO_SUBTREE (N_("File"), file_menu),
  GNOMEUIINFO_SUBTREE (N_("Data Set"), document_menu),
  GNOMEUIINFO_SUBTREE (N_("Preferences"), prefs_menu),
  /* FIXME in gnome-libs - need a macro for this. */
  {
    GNOME_APP_UI_JUSTIFY_RIGHT, 
    NULL, NULL, NULL, NULL, 
    NULL, 0, 0, '\0',
    0, NULL
  },
  GNOMEUIINFO_SUBTREE (N_("Help"), help_menu),
  GNOMEUIINFO_END
};

void 
GuppiAppImpl::make_menus()
{
  app_.create_menus_with_data(main_menu, this);

#ifdef GNOME_ENABLE_DEBUG
  g_print("Constructing import menu from registered importers:\n");
#endif
  ColumnImporterStore::const_iterator i = 
    global_column_importer_store.begin();
  size_t n = 0;

  while ( i != global_column_importer_store.end() ) {
#ifdef GNOME_ENABLE_DEBUG
    g_print("%d: %s\n", n, (*i)->importer_name().c_str());
#endif
    Gtk_MenuItem* mi = new Gtk_MenuItem((*i)->importer_name());
    gtk_menu_append(GTK_MENU(import_column_menu[0].widget),
		    GTK_WIDGET(mi->gtkobj()));

    connect_to_method(mi->activate,
		      this,
		      &GuppiAppImpl::import_file,
		      n);

    import_menuitems_.push_back(mi);
    mi->show();
    ++n;
    ++i;
  }
}

void
GuppiAppImpl::set_sensitivity()
{
  gtk_widget_set_sensitive(file_menu[MENU_SAVE_POS].widget,
			   doc_->need_save());  
}
