// -*- C++ -*-

/* 
 * axisedit.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include <gnome--.h>

#include "axisedit.h"


class AxisEditImpl : public Gtk_VBox {
public:
  AxisEditImpl(ColumnAxisDoc & cad,
	       GuppiDoc & gd,
	       bool removable);
  ~AxisEditImpl() { clear_optionmenu(); delete startadj_; delete stopadj_; }

  Signal0 remove;

private:
  void chose_column(const ColumnDoc* cd);

  Gtk_MenuItem* make_menuitem(const ColumnDoc & cd);
  void add_column(const ColumnDoc & cd);
  void delete_column(const ColumnDoc & cd);
  void change_column_name(const ColumnDoc & cd);

  void change_name(const AxisDoc & ad);
  void change_size(const AxisDoc & ad);

  void name_entry_activate();
  void start_changed();  
  void stop_changed();

  void show_size();
  
  // cruftiness because Gtk-- doesn't delete 
  //  children and Gtk+ doesn't allow
  //  easy modification of existing menus.
  void make_optionmenu();
  void clear_optionmenu();

  ColumnAxisDoc & cad_;
  GuppiDoc & gd_;

  Gtk_Entry      name_entry_;

  Gtk_OptionMenu optionmenu_;
  Gtk_Menu* menu_;

  list<Gtk_MenuItem*> menuitems_;

  Gtk_HBox       rangebox_;

  Gtk_Label      startlabel_, stoplabel_;

  // Have to be on the heap so we can change them
  Gtk_Adjustment* startadj_, * stopadj_;
  
  Gtk_SpinButton start_, stop_;

  // since we want to set the value and also respond to the 
  //  value being set by others, we need to switch off 
  //  the set when we change the spinbutton adjustment or 
  //  change the axisdoc
  bool changingstart_, changingstop_;

  Gtk_Button     remove_;
};

AxisEdit::AxisEdit(ColumnAxisDoc & cad,
		   GuppiDoc & gd, 
		   bool removable) 
  : Gtk_Frame(cad.get_name().c_str()), impl_(new AxisEditImpl(cad,gd,removable))
{
  set_label_align(0.1,0.0);
  set_shadow_type(GTK_SHADOW_IN);

  add(impl_);

  connect_to_method(impl_->remove,
		    this,
		    &AxisEdit::remove_cb);

}

AxisEdit::~AxisEdit()
{
  impl_->hide();
  delete impl_;
}

AxisEditImpl::AxisEditImpl(ColumnAxisDoc & cad,
			   GuppiDoc & gd, 
			   bool removable)
  : Gtk_VBox(TRUE,2), 
    
    cad_(cad),
    gd_(gd),
    
    name_entry_(),

    optionmenu_(),
    menu_(0),
    
    rangebox_(FALSE,2),
    
    startlabel_(_("Start")),
    stoplabel_(_("Stop")),
    
    startadj_(new Gtk_Adjustment(0.0,0.0,0.0)), 
    stopadj_ (new Gtk_Adjustment(0.0,0.0,0.0)),  
    
    start_(startadj_),
    stop_(stopadj_),
    
    changingstart_(false),
    changingstop_(false),

    remove_(_("Remove"))
{
  border_width(1);

  name_entry_.set_text(cad_.get_name());
  pack_start(&name_entry_,TRUE,TRUE);

  pack_start(&optionmenu_,TRUE,TRUE);

  make_optionmenu();

  pack_end(&rangebox_);

  rangebox_.pack_start(&startlabel_,FALSE,FALSE);
  rangebox_.pack_start(&start_,TRUE,TRUE);
  rangebox_.pack_end(&remove_,FALSE,FALSE);
  rangebox_.pack_end(&stop_,TRUE,TRUE);
  rangebox_.pack_end(&stoplabel_,FALSE,FALSE);

  // should be more intelligent.
  start_.set_digits(3);
  stop_.set_digits(3);

  show_size();

  remove_.set_sensitive(removable);

  connect_to_method(gd_.column_added,
		    this,
		    &AxisEditImpl::add_column);

  connect_to_method(gd_.column_deleted,
		    this,
		    &AxisEditImpl::delete_column);
  
  connect_to_signal(remove_.clicked,
		    remove);

  connect_to_method(cad_.name_changed,
		    this,
		    &AxisEditImpl::change_name);
		   
  connect_to_method(cad_.size_changed,
		    this,
		    &AxisEditImpl::change_size);

  connect_to_method(name_entry_.activate,
		    this,
		    &AxisEditImpl::name_entry_activate);

  connect_to_method(start_.changed,
		    this,
		    &AxisEditImpl::start_changed);

  connect_to_method(stop_.changed,
		    this,
		    &AxisEditImpl::stop_changed);

  // this behavior is really wrong for a standalone widget,
  // but should be OK
  show_all();
}

void 
AxisEditImpl::chose_column(const ColumnDoc* cd)
{
  cad_.set_column(*cd);
}

Gtk_MenuItem* 
AxisEditImpl::make_menuitem(const ColumnDoc & cd)
{
  Gtk_MenuItem* mi = new Gtk_MenuItem(cd.get_name().c_str());
  menu_->append(mi);
  menuitems_.push_back(mi);
  connect_to_method(mi->activate,
		    this,
		    &AxisEditImpl::chose_column,
		    &cd);
  connect_to_method(cd.name_changed,
		    this,
		    &AxisEditImpl::change_column_name);
  return mi;
}

void
AxisEditImpl::add_column  (const ColumnDoc & cd)
{
  Gtk_MenuItem* mi = make_menuitem(cd);
  mi->show();
}

void
AxisEditImpl::delete_column (const ColumnDoc & cd)
{
  clear_optionmenu();
  make_optionmenu();
}

void 
AxisEditImpl::change_column_name(const ColumnDoc& cd)
{
  // this can probably be done faster by passing 
  //  the column's menuitem as data and changing it.
  clear_optionmenu();
  make_optionmenu();
}

void 
AxisEditImpl::change_name(const AxisDoc & ad)
{
  name_entry_.set_text(ad.get_name().c_str());
}

void 
AxisEditImpl::change_size(const AxisDoc & ad)
{
  show_size();
}

void
AxisEditImpl::name_entry_activate()
{
  cad_.set_name(name_entry_.get_text());
}

void 
AxisEditImpl::start_changed()
{
  if (changingstart_) return;
  changingstart_ = true;
  gdouble v = start_.get_value_as_float();
  cad_.set_start(v);
  changingstart_ = false;
}

void 
AxisEditImpl::stop_changed()
{
  if (changingstop_) return;
  changingstop_ = true;
  gdouble v = stop_.get_value_as_float();
  cad_.set_stop(v);
  changingstop_ = false;
}

void
AxisEditImpl::show_size()
{
  if (changingstart_ || changingstop_) return;

  // hmm, perhaps we only need one of these.
  changingstart_ = true;
  changingstop_ = true;

  // fixme not sure the temporaries
  //  are required.

  // fixme this is kind of not smart enough at picking numbers
  Gtk_Adjustment* tmp1 = new Gtk_Adjustment(cad_.get_start(), cad_.get_min(),
					     cad_.get_max(), 1.0, 5.0, 1.0);

  Gtk_Adjustment* tmp2 = new Gtk_Adjustment(cad_.get_start(), cad_.get_min(),
					    cad_.get_max(), 1.0, 5.0, 1.0);
  
  start_.set_adjustment(tmp1);
  stop_.set_adjustment(tmp2);

  delete startadj_;
  delete stopadj_;

  startadj_ = tmp1;
  stopadj_  = tmp2;

  start_.set_value(cad_.get_start());
  stop_.set_value(cad_.get_stop());

  changingstart_ = false;
  changingstop_ = false;
}

void
AxisEditImpl::make_optionmenu()
{
  g_return_if_fail(menu_ == 0);

  menu_ = new Gtk_Menu;

  // The option menu destructor nukes
  //  the Gtk_Menu
  optionmenu_.set_menu(menu_);

  int h = 0; 
  GuppiDoc::const_column_iterator i = gd_.columns_begin();
  const ColumnDoc* current_column = cad_.get_column();
  if (current_column == 0) {
    if (i != gd_.columns_end()) {
      // set to the first column.
      current_column = *i;
      cad_.set_column(*current_column);
    }
    // Note that the axis can still get by without a column
    //  if there are no columns in the GuppiDoc.
    // This is fine but we always have to check the return
    //  value of get_column()
  }

  g_print("Current column %p\n", current_column);

  while ( i != gd_.columns_end() ) {
    make_menuitem(**i); // purposely ignoring return value

    // fixme - is this safe? maybe use a string
    if (*i == current_column) {
      optionmenu_.set_history(h);
      g_print("Setting column menu history to %d\n", h);
    }

    ++h;
    ++i;
  }
  
  menu_->show_all();
}

void
AxisEditImpl::clear_optionmenu()
{
  // Maybe this will do the right thing. Maybe it will leak
  //  memory or segfault. hmm. I don't like Gtk-- memory handling
  
  menu_->hide();

  list<Gtk_MenuItem*>::iterator i;

  if (menuitems_.size() > 0) {
    i = menuitems_.begin();
    while ( i != menuitems_.end() ) {
      delete *i;
      ++i;
    }
  }

  optionmenu_.remove_menu();
  menu_ = 0;
}







