// -*- C++ -*-

/* 
 * axistarget.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "axistarget.h"
#include "gtkaxis.h"

class AxisDocLabeller : public Axis_Labeller {
public:
  AxisDocLabeller(Axis_Base* ab, AxisDoc& ad);
  ~AxisDocLabeller();
 
  virtual void label();

private:
  AxisDoc & ad_;
};

AxisDocLabeller::AxisDocLabeller(Axis_Base* ab, AxisDoc& ad)
  : Axis_Labeller(ab), ad_(ad)
{
  
}

AxisDocLabeller::~AxisDocLabeller()
{

}

void
AxisDocLabeller::label()
{
  gdouble midpoint = ad_.get_start() + (ad_.get_stop() - ad_.get_start())/2;

  // 40 is a stupid hardcode for now. fixme
  draw_label(midpoint, 40, ad_.get_name().c_str());
}

////////////////////////////////////////////

// Right now the AxisTarget is kind of dumb; it's ScatterPlot specific.
//  needs to be corrected.

class AxisTarget : public Gtk_Axis {
public:
  AxisTarget(const AxisDoc & ad, Axis_Base::alignment_t a);
  ~AxisTarget();

private:
  const AxisDoc& ad_;
  // labeller used by all 
  Axis_Labeller* al_;
  // type-specific labeller
  Axis_Labeller* type_al_;

  void redraw();

  void change_size(const AxisDoc& ad);
  void change_name(const AxisDoc& ad);

  AxisTarget();
};


AxisTarget::AxisTarget(const AxisDoc & ad,
		       Axis_Base::alignment_t a) 
  : Gtk_Axis(a), ad_(ad), al_(0)
{
  connect_to_method(ad_.size_changed,
		    this,
		    &AxisTarget::change_size);

  connect_to_method(ad_.name_changed,
		    this,
		    &AxisTarget::change_name);

  // fixme, depends on the type of data in the axis
  //  and the type of plot. Will have to be requested
  //  from the plot type proxy
  al_ = new AxisDocLabeller(this,ad_);
  type_al_ = new Numeric_Axis_Labeller(this);
}

AxisTarget::~AxisTarget()
{
  delete al_;
  delete type_al_;
}

void
AxisTarget::redraw()
{
  set_bounds(ad_.get_start(), ad_.get_stop());

  Gtk_Axis::redraw(); // sets drawing target and does background
  gc_save();
  gc()->set_foreground("black");

  al_->label(); 
  type_al_->label();

  gc_restore();
}

void
AxisTarget::change_size(const AxisDoc & ad)
{
  redraw();
}


void
AxisTarget::change_name(const AxisDoc & ad)
{
  redraw();
}

//////////////////////////////////////////////////////

class AxisTargetGroupImpl : public Gtk_Signal_Base {
public:
  AxisTargetGroupImpl(Gtk_Grid & grid, 
		      Axis_Base::alignment_t a,
		      ScatterAxisList & sal);
  ~AxisTargetGroupImpl();

  // it's unclear that these are needed since we now connect directly
  //  to the axis list.
  void add_axis(const AxisDoc& ad, size_t i);
  void add_axis(const AxisDoc& ad);

  void delete_axis(size_t i);
  
private:
  Gtk_Grid & grid_;
  Axis_Base::alignment_t alignment_;
  ScatterAxisList & sal_;

  // These all abstract row/column according to 
  //  the orientation; kinda bad. Not the C++ way.
  size_t size() const;
  void   grid_add();
  void   grid_delete(size_t i);
  void   grid_insert(Gtk_Widget*w, size_t i);

  void add_axis_cb(ColumnAxisDoc& cad, size_t i);
  void delete_axis_cb(ColumnAxisDoc& cad, size_t i);
};

AxisTargetGroup::AxisTargetGroup(Axis_Base::alignment_t a,
				 ScatterAxisList & sal)
  : impl_(new AxisTargetGroupImpl(*this,a,sal))
{
  
}

AxisTargetGroup::~AxisTargetGroup()
{
  delete impl_;
}

inline void 
AxisTargetGroup::add_axis(const AxisDoc& ad, size_t i)
{
  impl_->add_axis(ad,i);
}

inline void 
AxisTargetGroup::add_axis(const AxisDoc& ad)
{
  impl_->add_axis(ad);
}


inline void 
AxisTargetGroup::delete_axis(size_t i)
{
  impl_->delete_axis(i);
}

/////////////////////////////////////


AxisTargetGroupImpl::AxisTargetGroupImpl(Gtk_Grid& grid, 
					 Axis_Base::alignment_t a,
					 ScatterAxisList& sal)
  : grid_(grid), alignment_(a), sal_(sal)
{
  // Vertical axis always has a single column,
  // horizontal a single row. 
  switch (alignment_) {
  case Axis_Base::EAST:
  case Axis_Base::WEST:
    grid_.add_column();
    break;
  case Axis_Base::NORTH:
  case Axis_Base::SOUTH:
    grid_.add_row();
    break;
  }

  // Sync to current state.
  ScatterAxisList::const_iterator ai = sal_.begin();
  size_t aj = 0;
  while ( ai != sal_.end() ) {
    add_axis(**ai,aj);
    ++ai;
    ++aj;
  }


  connect_to_method(sal_.axis_added,
		    this,
		    &AxisTargetGroupImpl::add_axis_cb);

  connect_to_method(sal_.axis_deleted,
		    this,
		    &AxisTargetGroupImpl::delete_axis_cb);

}

AxisTargetGroupImpl::~AxisTargetGroupImpl()
{
  
}

void 
AxisTargetGroupImpl::add_axis(const AxisDoc& ad, size_t i)
{
  size_t newindex = size();

#ifdef GNOME_ENABLE_DEBUG
  if ( i != newindex ) {
    g_warning("Only appends currently supported for Gtk_Grid");
    return;
  }
#endif
  
  grid_add();
  
  Gtk_Widget* w = new AxisTarget(ad,alignment_);

  grid_insert(w,newindex);  
  w->show();
}

void 
AxisTargetGroupImpl::add_axis(const AxisDoc& ad)
{
  add_axis(ad,size());
}


void 
AxisTargetGroupImpl::delete_axis(size_t i)
{
  grid_delete(i);
}

// These two are necessary due to signal typesafety

void 
AxisTargetGroupImpl::add_axis_cb(ColumnAxisDoc& cad, size_t i)
{
  add_axis(cad);
}

void 
AxisTargetGroupImpl::delete_axis_cb(ColumnAxisDoc& cad, size_t i)
{
  delete_axis(i);
}

size_t 
AxisTargetGroupImpl::size() const { 
  switch (alignment_) {
  case Axis_Base::EAST:
  case Axis_Base::WEST:
    return grid_.rows();
    break;
  case Axis_Base::NORTH:
  case Axis_Base::SOUTH:
    return grid_.columns();
    break;
  default:
    g_assert_not_reached();
    return 0;
  };
}

void   
AxisTargetGroupImpl::grid_add()
{
  switch (alignment_) {
  case Axis_Base::EAST:
  case Axis_Base::WEST:
    grid_.add_row();
    break;
  case Axis_Base::NORTH:
  case Axis_Base::SOUTH:
    grid_.add_column();
    break;
  default:
    g_assert_not_reached();
  };
}

void   
AxisTargetGroupImpl::grid_delete(size_t i)
{
  switch (alignment_) {
  case Axis_Base::EAST:
  case Axis_Base::WEST:
    grid_.delete_row(i);
    break;
  case Axis_Base::NORTH:
  case Axis_Base::SOUTH:
    grid_.delete_column(i);
    break;
  default:
    g_assert_not_reached();
  };
}

void   
AxisTargetGroupImpl::grid_insert(Gtk_Widget*w, size_t i)
{
  switch (alignment_) {
  case Axis_Base::EAST:
  case Axis_Base::WEST:
    grid_.add(w,i,0);
    break;
  case Axis_Base::NORTH:
  case Axis_Base::SOUTH:
    grid_.add(w,0,i);
    break;
  default:
    g_assert_not_reached();
  };
}
