// -*- C++ -*-

/* 
 * columnlist.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "columnlist.h"

#include "columnwindow.h"

class ColumnListImpl : public Gtk_CList {
public:
  ColumnListImpl(GuppiDoc& gd);
  ~ColumnListImpl();

  void add_column(const ColumnDoc& cd);
  void delete_column(const ColumnDoc& cd);

  void display_column_icon(const ColumnDoc & c);
  void display_column_name(const ColumnDoc & c);
  void display_column(const ColumnDoc & c);
  void display_all_columns();

  gint button_press_event_impl(GdkEventButton* be);

  void column_info();
  void column_delete();
private:
  GuppiDoc& gd_;

  Gtk_Menu popup_;
  Gtk_MenuItem mi_info_;
  Gtk_MenuItem mi_delete_;

  // last one clicked.
  ColumnDoc* clicked_cd_;
};

ColumnList::ColumnList(GuppiDoc& gd)
  : impl_(new ColumnListImpl(gd))
{
  add(impl_);
  impl_->show();
  set_shadow_type(GTK_SHADOW_OUT);
}

ColumnList::~ColumnList()
{
  impl_->hide();
  delete impl_;
}

ColumnListImpl::ColumnListImpl(GuppiDoc& gd) 
  : Gtk_CList(6), gd_(gd),  popup_(), mi_info_(_("Info...")),
    mi_delete_(_("Delete")), clicked_cd_(0)
{
  popup_.append(&mi_info_);
  mi_info_.show();
  popup_.append(&mi_delete_);
  mi_delete_.show();

  connect_to_method(mi_info_.activate,
		    this,
		    &ColumnListImpl::column_info);

  connect_to_method(mi_delete_.activate,
		    this,
		    &ColumnListImpl::column_delete);

  column_titles_show();
  column_titles_passive();

  set_column_title(0,_("Column"));
  set_column_width(0,100);
  set_column_title(1,_("N"));
  set_column_title(2,_("Mean"));
  set_column_title(3,_("Std. Dev."));
  set_column_title(4,_("Maximum"));
  set_column_title(5,_("Minimum"));

  gint i = 1;
  while ( i < 5 ) {
    set_column_width(i,65);
    ++i;
  }

  set_policy(GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);

  connect_to_method(gd_.column_added,
		    this,
		    &ColumnListImpl::add_column);

  connect_to_method(gd_.column_deleted,
		    this,
		    &ColumnListImpl::delete_column);

  display_all_columns();
}

ColumnListImpl::~ColumnListImpl()
{

}

void 
ColumnListImpl::add_column(const ColumnDoc& cd)
{
  connect_to_method(cd.name_changed, this, 
		    &ColumnListImpl::display_column_name);
  display_column(cd);
}

void 
ColumnListImpl::delete_column(const ColumnDoc& cd)
{
  clear();
  display_all_columns();
}

gint
ColumnListImpl::button_press_event_impl(GdkEventButton* be)
{
  gint row;
  gint col;
  if (get_selection_info(be->x,be->y,&row,&col) <= 0) 
    return Gtk_CList::button_press_event_impl(be);

  clicked_cd_ = (ColumnDoc *)get_row_data(row);

  if (clicked_cd_ != 0) {
    g_print("Button press, column data %s\n", 
	    clicked_cd_->get_name().c_str());
    if (be->button != 3) {
      return Gtk_CList::button_press_event_impl(be); 
    }
    else {
      g_print("Doing popup\n");
      popup_.popup(0,0,0,0,be->button,be->time);
    }
    return TRUE;
  }
  else {
    g_warning("Null data on row %d", row);
    return Gtk_CList::button_press_event_impl(be);
  }
}

void 
ColumnListImpl::display_all_columns()
{
  freeze();
  GuppiDoc::column_iterator i = gd_.columns_begin();
  while ( i != gd_.columns_end() ) {
    add_column(**i);
    ++i;
  }
  thaw();
}

void
ColumnListImpl::display_column_name(const ColumnDoc & c)
{
  display_column(c);
}

// unfortunately you can't set icon and name separately.
void 
ColumnListImpl::display_column(const ColumnDoc & c)
{
  int pos;

  pos = find_row_from_data(&c);

  // If the number of columns in the CList increases this 
  //  has to increase as well.
  // I passionately hate string array interfaces. They make a mess
  //  like this one.
  gchar* text[6];
  text[0] = c.get_name().c_str();

  // Each one needs its own buffer, since we 
  //  need all the buffers at once to stuff into the
  //  CList.
  gchar sizebuf[100];
  g_snprintf(sizebuf,100,"%u",c.ds().size());
  text[1] = &sizebuf[0];

  gchar meanbuf[100];
  g_snprintf(meanbuf,100,"%f",c.ds().mean());
  text[2] = &meanbuf[0];

  gchar sdevbuf[100];
  g_snprintf(sdevbuf,100,"%f",c.ds().sdev());
  text[3] = &sdevbuf[0];

  gchar maxbuf[100];
  g_snprintf(maxbuf,100,"%f",c.ds().max());
  text[4] = &maxbuf[0];
 
  gchar minbuf[100];
  g_snprintf(minbuf,100,"%f",c.ds().min());
  text[5] = &minbuf[0];  

  if ( pos != -1 ) {
    // icon already exists for this ColumnDoc - reinsert with new 
    //  characteristics.
    freeze();
    remove_row(pos);
    insert_row(pos,text);
    thaw();
  }
  else {
    // no icon yet, so add one.
    pos = append(text);
  }

  set_row_data(pos,&c);
}


void
ColumnListImpl::column_info()
{
  g_return_if_fail(clicked_cd_ != 0);

  Gnome_Dialog* d = new ColumnWindow(*clicked_cd_);
  d->show();  
}

void 
ColumnListImpl::column_delete()
{
  g_return_if_fail(clicked_cd_ != 0);
  
  gd_.delete_column(clicked_cd_);
}
