// -*- C++ -*-

/* 
 * columnutils.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "columnutils.h"

#ifdef GNOME_ENABLE_DEBUG
#undef GNOME_ENABLE_DEBUG
#endif

static bool
identifyNumber(string s)
{
  size_t p = s.find_first_of("eE",0);

  // if we have an exponent, slice it off and check it
  if (p != s.npos) {
    string expo = s.substr(p);
    s.erase(p);

    if (expo.size() < 2)
      return false;
    
    // make sure that both the 1e10, 1e+10 and 1e-10 type notation
    // is handled correctly
    if (expo[1] == '-' || expo[1] == '+') {
      if (expo.size() < 3)
	return false;
    } else if (!isdigit(expo[1]))
      return false;

    // make sure that the exponent contains only digits
    if (expo.find_first_not_of("0123456789",2) != expo.npos)
      return false;
  }

  // the empty string is not a float
  if (s.size() == 0)
    return false;

  // check the mantissa
  bool seenDecPt = false;
  for(size_t i=0; i<s.size(); ++i) {
    char c = s[i];
    if (!(isdigit(c) ||
	  (c == '-' && i == 0) || // only allow negative in first place
	  (c == '.' && !seenDecPt))) // only one dec pt per mantissa
      return false;
    if (c == '.')
      seenDecPt = true;
  }
  return true;
}

static bool
identifyDate(string s)
{
  //  cerr << "I don't know how to identify a date yet!" << endl;
  return false;
}

static ColumnDoc::DataType
identify(const string& s)
{
  if (identifyDate(s))
    return ColumnDoc::DATE;
  else if (identifyNumber(s))
    return ColumnDoc::REAL;
  else 
    return ColumnDoc::UNKNOWN;
}

////////////////////////////////////////////////////////

// return true on success - exceptions would be nice.
static bool 
get_next_token(const string & separator,
	       const string & s,
	       // where in s to start looking
	       size_t start,
	       // filled in.
	       // Index of next character after this token
	       size_t & token_end,
	       // Character after next separator; if no separator,
	       //  == token_end + 1
	       size_t & next_token)
{
#ifdef GNOME_ENABLE_DEBUG
  g_print("  ::Finding next token\n");
#endif
  if (start >= s.size())     return false;
  if (separator.size() == 0) return false;

#ifdef GNOME_ENABLE_DEBUG
  g_print("  ::start < s.size(), separator '%s', starting at '%s'...\n",
	  separator.c_str(), 
	  s.substr(start,MIN(start+10,s.size())-start).c_str());
#endif

  token_end = s.find_first_of(separator,start);

  if (token_end == string::npos) {
#ifdef GNOME_ENABLE_DEBUG
    g_print("  ::Didn't find a separator; returning entire string\n");
#endif
    // didn't find a separator, but this may be the last token.
    token_end  = s.size();
    next_token = token_end + 1; // on next call causes return false
  }
  else {
    next_token = token_end + separator.size();
  }

#ifdef GNOME_ENABLE_DEBUG
  if (token_end > s.size()) {
    g_warning("Token end %d too large!", token_end);
    return false;
  }
  g_print("  ::Found token: '%s'\n", s.substr(start,token_end-start).c_str());
#endif
  return true;
}

static bool
real_value(const string& s, double & value)
{
  value = atof(s.c_str());
  return true;
}

static bool
date_value(const string& s, double & value)
{
  g_warning("date_value not implemented!");
  value = 0;
  return false;
}

static bool 
token_value(ColumnDoc::DataType t, const string & s, double & value)
{
#ifdef GNOME_ENABLE_DEBUG
  g_print("Getting the value of token: %s\n", s.c_str());
#endif

  // remove leading and trailing whitespace, if any
  size_t i,j;
  for(i=0;i<s.size() && isspace(s[i]);++i);
  for(j=s.size()-1;j>=i && isspace(s[j]);--j);
  string ss = s.substr(i,j-i+1);

  switch (t) {
  case ColumnDoc::REAL:
    return real_value(ss, value);
    break;
  case ColumnDoc::DATE:
    return date_value(ss, value);
    break;
  case ColumnDoc::UNKNOWN:
    return real_value(ss, value);
    break;
  default:
    g_warning("Unknown data type: %d", t);
    g_assert_not_reached();
    return false;
  };
}

static list<ColumnDoc*> *
import_columnar(const string & text,
		const TextImportParams & params,
		DataSet* column_data[])
{
  size_t row = 0;
  size_t pos = 0;
  size_t end   = 0;
  size_t next = 0;
  while ( get_next_token(params.row_separator,
			 text,
			 pos,
			 end,
			 next) ) {

    // Only the requested number
    if (params.limit_elements && row == params.num_elements) break;

#ifdef GNOME_ENABLE_DEBUG
    g_print("--->row: %d pos: %d end: %d next: %d size: %d\n",
	    row,pos,end,next,text.size());
    g_print("--->Entering column-parsing loop\n");
#endif

    size_t column = 0;
    size_t cpos = 0;
    size_t cend = 0;
    size_t cnext = 0;
    string rowtext = text.substr(pos,end-pos);
    while ( get_next_token(params.element_separator,
			   rowtext,
			   cpos,
			   cend,
			   cnext) ) {

      // only get the requested number
      if (column == params.get_columns()) break;

#ifdef GNOME_ENABLE_DEBUG
    g_print("--->col: %d cpos: %d cend: %d cnext: %d size: %d\n",
	    column,cpos,cend,cnext,rowtext.size());
#endif

      double value;
      if ( token_value(params.get_type(column),
		       rowtext.substr(cpos,cend-cpos),
		       value) ) {
	column_data[column]->add(value);
      }
      else {
	g_warning("--->Didn't understand value: %s",
		  rowtext.substr(cpos,cend-cpos).c_str());
      }

      cpos = cnext;
      ++column;
#ifdef GNOME_ENABLE_DEBUG
    g_print("--->Leaving column-parsing loop\n");
#endif
    }
    pos = next;
    ++row;
  }
#ifdef GNOME_ENABLE_DEBUG
  g_print("--->Parsed %d rows\n", 
	  row);
#endif
  
  list<ColumnDoc*>* results = new list<ColumnDoc*>;

  size_t i = 0;
  while (i < params.get_columns()) {
    results->push_back(new ColumnDoc(*column_data[i]));
    ++i;
  }
  return results;
}


list<ColumnDoc*> * 
import_text(const string & text, 
	    const TextImportParams & params)
{
  DataSet * column_data[params.get_columns()];

  size_t i = 0;
  while ( i < params.get_columns() ) {
    column_data[i] = new DataSet;
    column_data[i]->reserve(params.num_elements);
    ++i;
  }

  if (params.columnar) {
    return import_columnar(text,params,column_data);
  }
  else {
    g_warning("Non-columnar import unimplemented");
    return 0;
  }
}

// fill params with our best guess
void               
guess_import_parameters(const string & text,
			TextImportParams & params)
{
  g_warning("guess_import_parameters not implemented, filling in garbage");

  params.columnar          = true;
  params.element_separator = " ";
  params.row_separator     = "\n";
  params.num_elements      = 60;
  params.limit_elements    = true;
  params.set_columns(3);
  params.set_type(0,ColumnDoc::REAL);
  params.set_type(1,ColumnDoc::REAL);
  params.set_type(2,ColumnDoc::REAL);
}
