// -*- C++ -*-

/* 
 * guppi.cc 
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include <algo.h>
#include <fstream>

#include "guppi.h"
#include "app.h"

Guppi::Guppi() : widget_idle_connected_(false)
{

}

Guppi::~Guppi()
{

}

//#if 0

void
Guppi::new_doc(GuppiDoc* doc, const gchar* geometry)
{
  g_return_if_fail(doc != 0);

  add_doc(doc);
  GuppiApp* a = new GuppiApp(doc,
			     geometry);
  a->show();
}

// creates an empty document. (for now with test data)
void 
Guppi::new_doc()
{
  GuppiDoc* doc = new GuppiDoc;

#if 0  
  ifstream infile("testdata.guppi");
  if (!infile) {
    g_warning("couldn't open file, arrgh");
    delete doc;
    exit();
  }
  
  char buf[4097];
  string s;
  
  while (infile) {
    infile.read(buf,4096);
    buf[infile.gcount() + 1] = '\0'; 
    s += buf;
  }
  
  if (doc->unserialize(s) == -1) {
    g_warning("Parse error!");
  }
#endif
  
  new_doc(doc,0);
}

void
Guppi::close_doc(GuppiDoc* doc)
{
  g_return_if_fail(doc != 0);

  if (doc->need_save()) {
    g_warning("Doc not saved, oops!"); // FIXME
  }

  if (docs_.size() == 1) {
    // this is the last window
    exit();
  }

  remove_doc(doc);
  munch(doc);
}

void
Guppi::save_doc(GuppiDoc* doc)
{
  // FIXME serialize doc and dump to file.

  doc->saved();
}

void
Guppi::exit()
{
  // fixme - check for unsaved documents, do modal dialogs 
  //  for each.
  g_warning("Exit doesn't check for unsaved documents yet.");
  Gtk_Main::instance()->quit();
}

void
Guppi::about()
{
  const gchar * authors [] = { "Jon Trowbridge <trow@emccta.com>",
			       "Havoc Pennington <hp@pobox.com>",
			       NULL };

  // could be NULL, but it's OK.
  gchar * logo = gnome_pixmap_file("guppi.xpm");

  Gnome_About * about = 
    new Gnome_About("Guppi: The GNU/GNOME Useful Plot Production Interface",
		    VERSION,
		    "Copyright (C) 1998 EMC Capital Management Inc.,"
		    " GNU General Public License.",
		    authors,
		    _("Guppi allows you to create several different kinds of plots "
		      "for data visualization."),
		    logo );

  g_free(logo);

  about->show();
  add_dialog(about);
}

#if 0
void
Guppi::session_die()
{

}

void 
Guppi::session_save_state(Gnome_Client* client, gint phase,
			  GnomeRestartStyle save_style, gint shutdown,
			  GnomeInteractStyle interact_style,
			  gint fast, gpointer client_data)
{


}
#endif


void
Guppi::remove_doc(GuppiDoc* doc)
{
  list<GuppiDoc*>::iterator i;
  i = find(docs_.begin(), docs_.end(), doc);
  if (i != docs_.end()) {
    docs_.erase(i);
  }
#ifdef G_ENABLE_CHECKS
  else {
    g_warning("Didn't find doc in remove_doc\n");
  }
#endif
}

void
Guppi::munch(Gtk_Widget* w)
{
  g_return_if_fail(w != 0);

  if (!widget_idle_connected_) {
    connect_to_method(Gtk_Main::idle(), 
		      this,
		      &Guppi::widget_munchidle);
    widget_idle_connected_ = true;
  }

  widget_munchlist_.push_back(w);

  g_print("Widget %p pushed onto the munchlist\n",w);
}

gint 
Guppi::widget_munchidle()
{
  g_assert(widget_idle_connected_);

  list<Gtk_Widget*>::iterator i = widget_munchlist_.begin();

  while (i != widget_munchlist_.end()) {
    g_print("Widget %p being munched\n",*i);
    // this next line is why widgets don't work with munchlist
    (*i)->hide();
    delete *i;
    g_print("Widget %p was munched\n",*i);
    ++i;
  }
  widget_munchlist_.clear();

  widget_idle_connected_ = false;
  return 0; // causes disconnect
}

/*************************** 
 Globals
 **********/
			   
static list<gchar*> start_geometries;
static list<gchar*> start_files;

/*********************************
  Arg parsing
  *****************************/

enum {
  GEOMETRY_KEY = -1
};

static struct argp_option argp_options [] = {
  { 
    "geometry",     /* name */
    GEOMETRY_KEY,   /* key  */
    N_("GEOMETRY"), /* name of argument to option */
    0,              /* flags */
    N_("Where to put the window, and its size"), /* docs */
    1               /* option group for help message sorting */
  },
  { NULL, 0, NULL, 0, NULL, 0 },
};

static error_t
parse_an_arg (int key, char *arg, struct argp_state *state)
{
  switch (key){
    
  case GEOMETRY_KEY:
    start_geometries.push_back(arg);
    break;
    
  case ARGP_KEY_ARG:
    start_files.push_back(arg);

  case ARGP_KEY_END:
    break;
    
  default:
    return ARGP_ERR_UNKNOWN;
  }

  return 0;
}

static struct argp parser =
{
  argp_options, parse_an_arg, NULL, NULL, NULL, NULL, NULL
};

//#endif

Guppi big_fish;

int
main(int argc, char * argv[])
{
  argp_program_version = VERSION;

  // Initialize the i18n stuff
  bindtextdomain (PACKAGE, GNOMELOCALEDIR);
  textdomain (PACKAGE);

  Gnome_Main gtkmain(Guppi::APPNAME, &parser, argc, argv, 0, 0);

#if 0  
  // filename args ignored for now.
  list<gchar*>::const_iterator igeometries;
  igeometries = start_geometries.begin();
  while (igeometries != start_geometries.end()) {
    big_fish.new_app(*igeometries);
    ++igeometries;
  }
#endif

  big_fish.new_doc();

  gtkmain.run();

  exit(0);
}
