// -*- C++ -*-

/* 
 * guppidoc.h 
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifndef GUPPI_GUPPIDOC_H
#define GUPPI_GUPPIDOC_H

// A Guppi "Document," which consists of a dataset and some plots
//  showing data from the set. 

// In implementation, it consists of a set of PlotDoc and a set of 
//  ColumnDoc. PlotDoc in turn has LayerDoc elements. 

// DataColumn changes significance under this scheme (see
// columndoc.h), and DataSet basically gets absorbed by GuppiDoc which
// has much the same functionality.

// The GuppiDoc should not have any GUI elements. It is the "model",
//  while the GuppiApp is the "view" and "controller" for the entire
//  document. There are specialized views for the individual columns
//  and plots.

// My plan is to use Gtk-- signals to keep the GUI in sync with the 
//  data. 

#include "saveable.h"
#include "plotdoc.h"
#include "columndoc.h"

class GuppiDoc : public Saveable,
		 public Named,
		 public Iconed,
		 public Serializable,
		 public Gtk_Signal_Base
{
public:

  GuppiDoc();
  virtual ~GuppiDoc();

  // GuppiDoc "owns" all the sub-documents and has to delete them.
  void add_column(ColumnDoc & c);
  void delete_column(ColumnDoc* c);

  void add_plot(PlotDoc & p);
  void delete_plot(PlotDoc* p);

  virtual string   serialize() const;
  virtual int     unserialize(const string & s);

  ///////////////////////////// Since the GuppiDoc is sort 
  ///////////////////////////// of a double container this
  ///////////////////////////// looks ugly. 

  typedef list<ColumnDoc*>::iterator column_iterator;
  typedef list<PlotDoc*>::iterator   plot_iterator;

  typedef list<ColumnDoc*>::const_iterator const_column_iterator;
  typedef list<PlotDoc*>::const_iterator   const_plot_iterator;

  // FIXME these all have an extra copy for legacy reasons.

  column_iterator columns_begin() {
    column_iterator i(columns_.begin());
    return i;
  }

  column_iterator columns_end() {
    column_iterator i(columns_.end());
    return i;
  }
  
  plot_iterator plots_begin() {
    plot_iterator i(plots_.begin());
    return i;
  }
  
  plot_iterator plots_end() {
    plot_iterator i(plots_.end());
    return i;
  }

  const_column_iterator columns_begin() const {
    const_column_iterator i(columns_.begin());
    return i;
  }

  const_column_iterator columns_end() const {
    const_column_iterator i(columns_.end());
    return i;
  }
  
  const_plot_iterator plots_begin() const {
    const_plot_iterator i(plots_.begin());
    return i;
  }
  
  const_plot_iterator plots_end() const {
    const_plot_iterator i(plots_.end());
    return i;
  }

  ///////////////////// Signals 
  Signal1<const ColumnDoc &> column_added;
  Signal1<const ColumnDoc &> column_deleted;

  Signal1<const PlotDoc &>   plot_added;
  Signal1<const PlotDoc &>   plot_deleted;

  Signal1<const GuppiDoc &> name_changed;
  Signal1<const GuppiDoc &> icon_changed;

  Signal0 destroy;

  void column_data_changed(const DataSet & ds);

private:
  void named_callback(const string & name) {
    name_changed(*this);
  }

  void iconed_callback(const string & icon) {
    icon_changed(*this);
  }

  // Stored under their names
  // In retrospect this sucks in many ways; fixme
  list<ColumnDoc*> columns_;
  list<PlotDoc*> plots_;
  
  // Number of documents so far, used in the 
  //  default title.
  static guint num_docs_ = 0;
};


#endif
