// -*- C++ -*-

/* 
 * plotwindow.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "plotwindow.h"
#include "guppi.h"

class PlotWindowImpl : public Gtk_VBox {
public:
  PlotWindowImpl(const PlotDoc & pd, PlotWindow & pw);
  ~PlotWindowImpl();

  PlotWindow* pw() { return &pw_; }

private:
  void make_menus();

  void add_plot(Plot& plot);

  // Callbacks to keep up with the PlotDoc
  void display_name(const PlotDoc & pd);
  void display_icon(const PlotDoc & pd);

#if 0  
  void move_layer(const LayerDoc & ld);
  void add_layer(const LayerDoc & ld);
  void delete_layer(const LayerDoc & ld);
#endif

  void coordinates_status_display(gdouble x, gdouble y,
				  gdouble w, gdouble h);
  void rubberband_status_display(gdouble x_origin, gdouble y_origin,
				 gdouble x, gdouble y);


  const PlotDoc& pd_;
  PlotWindow& pw_;

  Gnome_AppBar appbar_;

  Plot* plot_;
};


PlotWindow::PlotWindow(const PlotDoc& pd) : 
  Gnome_App(Guppi::APPNAME, ""), 
  impl_(new PlotWindowImpl(pd,*this))
{
  connect_to_method(pd.destroy, this,
		    &PlotWindow::self_munch);
  impl_->show();
}

PlotWindow::~PlotWindow()
{
  impl_->hide();
  delete impl_;
#ifdef GNOME_ENABLE_DEBUG
  g_print("~PlotWindow\n");
#endif
}


void
PlotWindow::self_munch()
{
  big_fish.munch(this);
}


PlotWindowImpl::PlotWindowImpl(const PlotDoc& pd, PlotWindow& pw)
  :  pd_(pd), pw_(pw),
     appbar_(true,true,GNOME_PREFERENCES_USER),
     plot_(0)
{
  make_menus();
  pw_.set_contents(this);
  pw_.set_statusbar(&appbar_);

  connect_to_method(pd_.name_changed,
		    this,
		    &PlotWindowImpl::display_name);

  add_plot(*pd_.make_plot());  
}

PlotWindowImpl::~PlotWindowImpl()
{
  if (plot_) {
    plot_->hide();
    delete plot_;
  }
}

void
PlotWindowImpl::add_plot(Plot& plot)
{
  g_return_if_fail(plot_ == 0);

  plot_ = &plot;
  pack_start(&plot,TRUE,TRUE);

  connect_to_method(plot.coordinates,this,
		    &PlotWindowImpl::coordinates_status_display);
  connect_to_method(plot.rubberband_coordinates,this,
		    &PlotWindowImpl::rubberband_status_display);
  plot.show();
}

void
PlotWindowImpl::display_name(const PlotDoc & pd)
{
  pw_.set_title(pd.get_name().c_str());
}

void
PlotWindowImpl::display_icon(const PlotDoc & pd)
{

}

#if 0
void
PlotWindowImpl::move_layer(const LayerDoc & ld)
{

}

void
PlotWindowImpl::add_layer(const LayerDoc & ld)
{

}

void
PlotWindowImpl::delete_layer(const LayerDoc & ld)
{

}
#endif


void 
PlotWindowImpl::coordinates_status_display(gdouble x, gdouble y,
				       gdouble w, gdouble h)
{
  gchar buf[100];
  gchar * wprecision = ( (w < 10.0) && (w > -10.0) ) ? "%f" : "%.2f";
  gchar * hprecision = ( (h < 10.0) && (h > -10.0) ) ? "%f" : "%.2f";
  gchar * format = g_copy_strings(wprecision," x ", hprecision, ": ", 
				  wprecision,",",hprecision,NULL);
  g_snprintf(buf,100,format, w, h, x, y); 
  appbar_.set_status(buf);
  g_free(format);
}

void 
PlotWindowImpl::rubberband_status_display(gdouble x_origin, gdouble y_origin,
				      gdouble x, gdouble y)
{
  gchar buf[100];
  gdouble dx = x_origin - x;
  gdouble dy = y_origin - y;
  gchar * xprecision = (dx < 10.0 && dx > -10.0) ? "%f" : "%.2f";
  gchar * yprecision = (dy < 10.0 && dy > -10.0) ? "%f" : "%.2f";
  gchar * format = g_copy_strings("(",xprecision,",", yprecision, ") (", 
				  xprecision,",",yprecision,")",NULL);
  g_snprintf(buf,100,format,x_origin,y_origin, x, y); 
  appbar_.set_status(buf);
  g_free(format);
}


///////////////////////////////// Menu cruft

// Bunch of callbacks - they just call the relevant methods on the 
//  object sent in as data.

static void 
about_cb(GtkWidget * w, gpointer data)
{
  g_return_if_fail(data != 0);
  big_fish.about();
}

static void
close_cb(GtkWidget * w, gpointer data)
{
  g_return_if_fail(data != 0);
  // app can't close itself.
  big_fish.munch(((PlotWindowImpl *)data)->pw());
}

static void 
exit_cb(GtkWidget * w, gpointer data)
{
  big_fish.exit();
}

static void 
view_cb(GtkWidget * w, gpointer data)
{

}

static void
preferences_cb(GtkWidget * w, gpointer data)
{
  g_return_if_fail(data != 0);

}


static GnomeUIInfo help_menu[] = {
  GNOMEUIINFO_HELP(Guppi::APPNAME),
  {GNOME_APP_UI_ITEM, N_("About..."), 
   N_("Tell about this application"), 
   about_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT, 0, 0, NULL },
  GNOMEUIINFO_END
};

static GnomeUIInfo file_menu[] = {
  {GNOME_APP_UI_ITEM, N_("Close"), 
   N_("Close this window"),
   close_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXIT, 'c', 
   GDK_CONTROL_MASK, NULL },
  GNOMEUIINFO_SEPARATOR,
  {GNOME_APP_UI_ITEM, N_("Exit"), 
   N_("Quit the application"),
   exit_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXIT, 'q', 
   GDK_CONTROL_MASK, NULL },
  GNOMEUIINFO_END
};

static GnomeUIInfo view_menu[] = {
  {GNOME_APP_UI_TOGGLEITEM, N_("Some menu item"), 
   N_("Does something or other"),
   view_cb, NULL, NULL,
   0, 0, 'f', 
   GDK_CONTROL_MASK, NULL },
  GNOMEUIINFO_END
};

static GnomeUIInfo prefs_menu[] = {
  {GNOME_APP_UI_ITEM, N_("Preferences..."), 
   N_("Change application preferences"),
   preferences_cb, NULL, NULL,
   GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PREF, 'p', 
   GDK_CONTROL_MASK, NULL },
  GNOMEUIINFO_END
};

static GnomeUIInfo main_menu[] = {
  GNOMEUIINFO_SUBTREE (N_("File"), file_menu),
  GNOMEUIINFO_SUBTREE (N_("View"), view_menu),
  GNOMEUIINFO_SUBTREE (N_("Preferences"), prefs_menu),
  /* FIXME in gnome-libs - need a macro for this. */
  {GNOME_APP_UI_JUSTIFY_RIGHT, 
   NULL, NULL, NULL, NULL, 
   NULL, 0, 0, '\0',
   0, NULL},
  GNOMEUIINFO_SUBTREE (N_("Help"), help_menu),
  GNOMEUIINFO_END
};

void 
PlotWindowImpl::make_menus()
{
  pw_.create_menus_with_data(main_menu, this);
}



