// -*- C++ -*-

/* 
 * scatterdoc.cc 
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */


#include "scatterdoc.h"
#include "scatteredit.h"
#include "scatterplot.h"

ScatterAxisList::~ScatterAxisList()
{
  clear();
}

void
ScatterAxisList::clear()
{
  size_t i = 0;
  size_t n = axes_.size();
  while ( i < n ) {
    ColumnAxisDoc* cad = axes_[i];
    axis_deleted(*cad,i);
    delete cad;
    ++i;
  }
}

SingleScatter::SingleScatter() : 
  x_axis_(0), 
  y_axis_(0),
  marker_(Plot_Area::TIMES)
{
  
}

SingleScatter::SingleScatter(const ColumnAxisDoc& xcad,
			     const ColumnAxisDoc& ycad) :
  x_axis_(&xcad),
  y_axis_(&ycad),
  marker_(Plot_Area::DOT)
{
}

SingleScatter::~SingleScatter() 
{
#ifdef GNOME_ENABLE_DEBUG
  g_print("Deleting SingleScatter %p\n", this);
#endif
  destroy();
}

ScatterDoc::ScatterDoc() 
{
  connect_to_method(x_axes.axis_added,
		    this,
		    &ScatterDoc::add_column);
  connect_to_method(x_axes.axis_deleted,
		    this,
		    &ScatterDoc::delete_column);

  connect_to_method(y_axes.axis_added,
		    this,
		    &ScatterDoc::add_row);
  connect_to_method(y_axes.axis_deleted,
		    this,
		    &ScatterDoc::delete_row);
}

ScatterDoc::~ScatterDoc()
{
  size_t i = 0;
  size_t r = singles_.rows();
  while ( i < r ) {
    size_t j = 0;
    size_t c = singles_.columns();
    while ( j < c ) {
      delete singles_[i][j];
      ++j;
    }
    ++i;
  }
}

Gtk_Widget*
ScatterDoc::make_editor(GuppiDoc & gd)
{
  return new ScatterEdit(*this,gd);
}

Plot*
ScatterDoc::make_plot() const
{
  return new ScatterPlot(*this);
}

void
ScatterDoc::add_row(ColumnAxisDoc & cad, size_t i)
{
  g_print("About to add row.\n");

  singles_.add_row();

  size_t j = 0;
  size_t c = singles_.columns();
  ScatterAxisList::const_iterator k = x_axes.begin();
  while ( j < c ) {
    if (k == x_axes.end()) g_warning("ScatterDoc::add_row: Not enough axes!");
    singles_[i][j] = new SingleScatter(**k,cad);
    g_print("Adding new SingleScatter %d,%d @ %p\n", i,j,singles_[i][j]);
    single_added(*singles_[i][j],i,j);
    ++j;
    ++k; 
  }
  
  row_added(i);

#ifdef GNOME_ENABLE_DEBUG
  spew();
  g_print("Added.\n");
#endif
}

void
ScatterDoc::add_column(ColumnAxisDoc & cad, size_t i)
{
  g_print("About to add column.\n");

  singles_.add_column();

  size_t j = 0;
  size_t r = singles_.rows();
  ScatterAxisList::const_iterator k = y_axes.begin();

  while ( j < r ) {
    if (k == y_axes.end()) 
      g_warning("ScatterDoc::add_column: Not enough axes!");
    singles_[j][i] = new SingleScatter(cad,**k);
    g_print("Adding new SingleScatter %d,%d @ %p\n", j,i,singles_[j][i]);
    single_added(*singles_[j][i],j,i);
    ++j;
    ++k;
  }

  column_added(i);

#ifdef GNOME_ENABLE_DEBUG
  spew();  
  g_print("Added.\n");
#endif
}

void
ScatterDoc::delete_row(ColumnAxisDoc & cad, size_t i)
{
#ifdef GNOME_ENABLE_DEBUG
  g_print("About to delete row.\n");
  spew();
#endif  

  row_deleted(i);

  size_t j = 0;
  size_t c = singles_.columns();
  
  while ( j < c ) {
    single_deleted(*singles_[i][j],i,j);
    delete singles_[i][j];
    g_print("Deleted SingleScatter %d,%d @ %p\n", i,j,singles_[i][j]);
    ++j;
  }

  singles_.delete_row(i);

#ifdef GNOME_ENABLE_DEBUG
  g_print("Deleted.\n");
  spew();
#endif
}

void
ScatterDoc::delete_column(ColumnAxisDoc & cad, size_t i)
{
#ifdef GNOME_ENABLE_DEBUG
  g_print("About to delete column.\n");
  spew();
#endif

  column_deleted(i);

  size_t j = 0;
  size_t r = singles_.rows();
  
  while ( j < r ) {
    single_deleted(*singles_[j][i],j,i);
    delete singles_[j][i];
    g_print("Deleted SingleScatter %d,%d @ %p\n", j,i,singles_[j][i]);
    ++j;
  }

  singles_.delete_column(i);

#ifdef GNOME_ENABLE_DEBUG
  g_print("Deleted.\n");
  spew();
#endif
}

#ifdef GNOME_ENABLE_DEBUG
void 
ScatterDoc::spew()
{
  size_t r = singles_.rows();
  size_t c = singles_.columns();

  g_print("+======= %d x %d\n",r,c);
  size_t i = r;
  while ( i > 0 )  {
    --i;

    g_print("| Row %5u: ", i);
    size_t j = 0;
    while ( j < c ) {
      g_print(" [%p]", singles_[i][j]);
      ++j;
    }
     g_print("\n");
  };
  g_print("+=======\n");
}
#endif

