// -*- C++ -*-

/* 
 * scatterdoc.h 
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifndef GUPPI_SCATTERDOC_H
#define GUPPI_SCATTERDOC_H

#include "plotdoc.h"
#include "axisdoc.h"
#include "colored.h"
#include "grid.h"

class ScatterAxisList {
public:
  ScatterAxisList() : spacing_(0) {}
  ~ScatterAxisList();

  typedef Array<ColumnAxisDoc*>::const_iterator const_iterator;

  const_iterator begin() const { return axes_.begin(); }
  const_iterator end() const { return axes_.end(); }
  
  // The ScatterDoc now owns the axis for memory management.
  void push_back(ColumnAxisDoc & cad) {
    size_t n = axes_.size();
    axes_.push_back(&cad); 
    axis_added(cad,n);
  }

  void clear();

  // Hoses any existing iterators!
  void delete_axis(size_t i) {
    axis_deleted(*(axes_[i]), i);
    delete axes_[i];
    axes_.erase(i);
  }

  size_t size() const { return axes_.size(); }

  Signal2<ColumnAxisDoc&, size_t> axis_added;
  Signal2<ColumnAxisDoc&, size_t> axis_deleted;  

  // Spacing between plots.
  guint    get_spacing() const { return spacing_; }
  void    set_spacing(guint s) { spacing_ = s; spacing_changed(); }
  Signal0 spacing_changed;

private:
  Array<ColumnAxisDoc*> axes_;
  guint spacing_;
};


/////////////////////////////////////////////////////////////////////

// FIXME this needs to be a full Doc? Not really I guess.

// A single plot in the grid of plots.
class SingleScatter : public Colored
{
public:
  SingleScatter(const ColumnAxisDoc& xcad,
		const ColumnAxisDoc& ycad);
  SingleScatter();
  ~SingleScatter();

  void set_axes(const ColumnAxisDoc& xcad,
		const ColumnAxisDoc& ycad);

  const ColumnAxisDoc& get_x_axis() const { 
#ifdef GNOME_ENABLE_DEBUG
    if (x_axis_ == 0) { g_warning("No X axis in SingleScatter"); }
#endif
    return *x_axis_; 
  }

  const ColumnAxisDoc& get_y_axis() const { 
#ifdef GNOME_ENABLE_DEBUG
    if (y_axis_ == 0) { g_warning("No Y axis in SingleScatter"); }
#endif
    return *y_axis_; 
  }
  
  Plot_Area::marker_t get_marker() const { return marker_; }
  void set_marker(Plot_Area::marker_t m) { 
    marker_ = m; marker_changed(*this); 
  }

  mutable Signal1<const SingleScatter&> color_changed;
  mutable Signal1<const SingleScatter&> marker_changed;
  Signal0 destroy;

private:
  const ColumnAxisDoc* x_axis_;
  const ColumnAxisDoc* y_axis_;
  
  Plot_Area::marker_t marker_;

  void color_changed_callback(const Colored& c) {
    color_changed(*this);
  }
};

///////////////////////////////////////////////////////////////////

class ScatterDoc : public PlotDoc
{
public:
  ScatterDoc();
  ~ScatterDoc();

  // These aren't really data members; the ScatterAxisList 
  //  has the interface. You can use it to get the 
  //  number of axes each way and iterate over the 
  //  axes.
  ScatterAxisList x_axes;
  ScatterAxisList y_axes;

  // Sure wish one could overload operator[][]
  SingleScatter& get_single(size_t row, size_t col) { 
    return *singles_[row][col];
  }
  const SingleScatter& get_single(size_t row, size_t col) const {
    return *singles_[row][col];
  }

  size_t rows() const { return singles_.rows(); }
  size_t columns() const { return singles_.columns(); }

  // Not sure these are useful, they can be deleted later on.
  mutable Signal3<const SingleScatter&, size_t, size_t> single_added;
  mutable Signal3<const SingleScatter&, size_t, size_t> single_deleted;

  Signal1<size_t> column_added;
  Signal1<size_t> column_deleted;
  Signal1<size_t> row_added;
  Signal1<size_t> row_deleted;

  virtual string   serialize() const { return ""; }
  virtual int      unserialize(const string & s) { return  0; }

  virtual Gtk_Widget* make_editor(GuppiDoc & gd);
  virtual Plot*       make_plot  () const;

private:
  Grid<SingleScatter*> singles_;

  void    add_row(ColumnAxisDoc &, size_t i);
  void    add_column(ColumnAxisDoc &, size_t i);

  void    delete_row(ColumnAxisDoc &, size_t i);
  void    delete_column(ColumnAxisDoc &, size_t i);

#ifdef GNOME_ENABLE_DEBUG
  void    spew();
#endif
};


#endif


