// -*- C++ -*-

/* 
 * scatteredit.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "scatteredit.h"
#include "axisedit.h"
#include "mathutil.h"
#include "markermenu.h"
#include "guppi.h"
#include "gtkgrid.h"

class AxisEditGroup : public Gtk_Frame {
public:
  AxisEditGroup(ScatterAxisList & sal,
		GuppiDoc & gd, const gchar* title,
		const gchar* axisedit_title_format,
		const gchar* more_label,
		GtkOrientation orientation);
  ~AxisEditGroup();
private:
  ScatterAxisList & sal_;
  GuppiDoc & gd_;

  Gtk_VBox     box_;

  Gtk_Box*     extra_; // hold additional axes

  // add more axes.
  Gtk_Button   more_;

  // contents of button
  Gtk_HBox     more_box_;
  Gtk_Label    more_label_;
  Gtk_Arrow    more_arrow_;

  list<AxisEdit*> axis_edits_;

  const gchar* title_format_;

  GtkOrientation orientation_;

  void label_axisedit(AxisEdit& ae,gint n);
  void add_axisedit(ColumnAxisDoc& cad);
  void delete_axisedit(AxisEdit* ae);
  void more_clicked();
  void remove_clicked(AxisEdit* ae);

  void add_axis(ColumnAxisDoc& cad, size_t i);
  void delete_axis(ColumnAxisDoc& cad, size_t i);
};

AxisEditGroup::AxisEditGroup(ScatterAxisList & sal,
			     GuppiDoc & gd, const gchar* title,
			     const gchar* axisedit_title_format,
			     const gchar* more_label,
			     GtkOrientation orientation)
  : Gtk_Frame(title),

    sal_(sal),
    gd_(gd),
    
    box_ (FALSE,GNOME_PAD),

    extra_(0),
    
    more_(),  

    more_box_(FALSE,2), 
    more_label_(more_label),
    more_arrow_(GTK_ARROW_DOWN,GTK_SHADOW_OUT),

    title_format_(axisedit_title_format),
    orientation_(orientation)
{
  set_label_align(0.5,0.0);

  switch (orientation_) {
  case GTK_ORIENTATION_VERTICAL:
    extra_ = new Gtk_VBox(TRUE,GNOME_PAD);
    break;
  case GTK_ORIENTATION_HORIZONTAL:
    extra_ = new Gtk_HBox(TRUE,GNOME_PAD);
    break;
  default:
    g_assert_not_reached();
  }

  box_.border_width(GNOME_PAD);
  add(&box_);

  box_.pack_end(&more_,FALSE,FALSE);
  box_.pack_end(extra_,TRUE,TRUE);
  more_.add(&more_box_);
  more_box_.pack_start(&more_label_,TRUE,TRUE);
  more_box_.pack_end(&more_arrow_,FALSE,FALSE);

  connect_to_method(more_.clicked,
		    this,
		    &AxisEditGroup::more_clicked);

  connect_to_method(sal_.axis_added,
		    this,
		    &AxisEditGroup::add_axis);

  connect_to_method(sal_.axis_deleted,
		    this,
		    &AxisEditGroup::delete_axis);

  ScatterAxisList::const_iterator i = sal_.begin();

  while ( i != sal_.end() ) {
    add_axisedit(**i);
    ++i;
  }
}

AxisEditGroup::~AxisEditGroup()
{
  list<AxisEdit*>::iterator i = axis_edits_.begin();
  
  while ( i != axis_edits_.end() ) {
    (*i)->hide();
    delete *i;
    ++i;
  }

  extra_->hide();
  delete extra_;
}

void 
AxisEditGroup::label_axisedit(AxisEdit& ae, gint n)
{
  size_t len = strlen(title_format_) + MAX_INT_STRING;
  char* buf = new char[len];

  g_snprintf(buf,len,title_format_,n);

  ae.set_label(buf);

  delete buf;
}

void 
AxisEditGroup::add_axisedit(ColumnAxisDoc& cad)
{
  AxisEdit* ae = new AxisEdit(cad,gd_);
  
  label_axisedit(*ae, (gint) axis_edits_.size() + 1);
  
  connect_to_method(ae->remove,
		    this,
		    &AxisEditGroup::remove_clicked);

  connect_to_method(cad.destroy,
		    this,
		    &AxisEditGroup::delete_axisedit,
		    ae);

  switch (orientation_) {
  case GTK_ORIENTATION_VERTICAL:
    extra_->pack_end(ae,TRUE,TRUE);
    break;
  case GTK_ORIENTATION_HORIZONTAL:
    extra_->pack_start(ae,TRUE,TRUE);
    break;
  default:
    g_assert_not_reached();
  }

  ae->show();
  axis_edits_.push_back(ae);  
}

void 
AxisEditGroup::more_clicked()
{
  ColumnAxisDoc* cad = new ColumnAxisDoc;
  sal_.push_back(*cad);
}

void
AxisEditGroup::delete_axisedit(AxisEdit* ae)
{
  // I think lists should have find(). pttht.
  //  I suppose a tree would be better.
  list<AxisEdit*>::iterator i = axis_edits_.begin();

  while ( i != axis_edits_.end() ) {
    if ( *i == ae ) break;
    ++i;
  }

  if (i == axis_edits_.end()) g_warning("Axis edit not found!");

  axis_edits_.erase(i);

  extra_->remove(ae);
  ae->hide();

  queue_resize();

  // Redo the numbering.
  int n = 1;
  i = axis_edits_.begin();
  while (i != axis_edits_.end()) {
    label_axisedit(**i, n);
    ++n;
    ++i;
  }
}

void
AxisEditGroup::remove_clicked(AxisEdit* ae)
{
  list<AxisEdit*>::iterator i = axis_edits_.begin();
  size_t j = 0;

  while ( i != axis_edits_.end() ) {
    if ( *i == ae ) break;
    ++i;
    ++j; 
  }

  if (i == axis_edits_.end()) g_warning("Axis edit not found!");

  // results in a call to delete_axisedit via destroy
  //  method of the deleted ColumnAxisDoc
  // The above loop thus has to run twice, kind of silly. This is 
  // all a big mess.
  sal_.delete_axis(j);
}

void 
AxisEditGroup::add_axis(ColumnAxisDoc& cad, size_t i)
{
  // doesn't work unless it's an append - fixme
  add_axisedit(cad);
}

void 
AxisEditGroup::delete_axis(ColumnAxisDoc& cad, size_t i)
{
  g_warning("AxisEditGroup::delete_axis is probably useless");
  // don't care; destroy of ColumnAxisDoc signals deletion
}

////////////////////////////////////////////////////////////////////

// blech aargh
void color_set_func(GnomeColorSelector* gcs,gpointer data)
{
  SingleScatter* ss = (SingleScatter*)data;
  gint r,g,b;
  
  gnome_color_selector_get_color_int(gcs,&r,&g,&b,0xFFFF);

  ss->set_color_rgb(r,g,b);
}

class SingleEdit : public Gtk_Frame {
public:
  SingleEdit(SingleScatter& ss);
  ~SingleEdit();

private:
  SingleScatter& ss_;

  Gtk_VBox box_;
  Gtk_Label name_;
  Gtk_OptionMenu markeroption_;
  MarkerMenu mm_;
  Gtk_HBox   markerbox_;
  Gtk_Label  markerlabel_;
  Gnome_ColorSelector markercolor_;
  //Gnome_ColorSelector bgcolor_;
  

  void marker_chosen(Plot_Area::marker_t);
  void change_marker(const SingleScatter & ss);
  void change_name(const AxisDoc& ad);
  void display_name();
  void ss_destroyed();

  SingleEdit();
};

SingleEdit::SingleEdit(SingleScatter& ss) 
  : ss_(ss), box_(FALSE,GNOME_PAD_SMALL), name_(""), 
    markeroption_(), mm_(),
    markerbox_(FALSE,GNOME_PAD_SMALL), 
    markerlabel_(_("Marker color: ")),
    markercolor_(color_set_func,&ss)
{
  set_shadow_type(GTK_SHADOW_IN);
  add(&box_);
  box_.pack_start(&name_,FALSE,FALSE);
  name_.show();
  box_.pack_start(&markeroption_,FALSE,FALSE);
  markeroption_.show();
  box_.show();

  markeroption_.set_menu(&mm_);

  change_marker(ss);

  display_name();

  connect_to_method(ss_.marker_changed,
		    this,
		    &SingleEdit::change_marker);

  connect_to_method(mm_.marker_activated,
		    this,
		    &SingleEdit::marker_chosen);

  connect_to_method(ss_.get_x_axis().name_changed,
		    this,
		    &SingleEdit::change_name);

  connect_to_method(ss_.get_y_axis().name_changed,
		    this,
		    &SingleEdit::change_name);

  connect_to_method(ss_.destroy,
		    this,
		    &SingleEdit::ss_destroyed);

  markerbox_.pack_start(&markerlabel_,TRUE,TRUE);
  markerbox_.pack_end(&markercolor_.get_button(),FALSE,FALSE);
  box_.pack_start(&markerbox_,FALSE,FALSE);
  markerbox_.show_all();
}

SingleEdit::~SingleEdit()
{
  markeroption_.remove_menu();
}

void 
SingleEdit::marker_chosen(Plot_Area::marker_t m)
{

  ss_.set_marker(m);
}

void 
SingleEdit::change_marker(const SingleScatter & ss)
{
  markeroption_.set_history(ss.get_marker());
}

void 
SingleEdit::change_name(const AxisDoc& ad)
{
  display_name();
}

void 
SingleEdit::display_name()
{
  string label = ss_.get_x_axis().get_name() + " vs. " + 
    ss_.get_y_axis().get_name();
  name_.set(label);
}

void
SingleEdit::ss_destroyed()
{
  big_fish.munch(this);
}

////////////////////////////////////////////////////////////////////

class SingleEditGroup : public Gtk_Frame {
public:
  SingleEditGroup(ScatterDoc &sd);
  ~SingleEditGroup();
  
private:
  ScatterDoc& sd_;

  Gtk_Grid grid_;

  Gtk_Label no_plots_;

  bool grid_added_;

  void add_row(size_t r);
  void delete_row(size_t r);
  void add_column(size_t c);
  void delete_column(size_t c);

  void install_grid();
  void install_noplots();

  SingleEditGroup();
};

SingleEditGroup::SingleEditGroup(ScatterDoc& sd)
  : sd_(sd), grid_(), no_plots_(_("No plots.")), grid_added_(false)
{
  size_t r = sd.rows();
  size_t c = sd.columns();

  grid_.reserve(r,c);
  
  if (r == 0 || c == 0) {
    add(&no_plots_);
    no_plots_.show();
    grid_added_ = false;
  }
  else {
    size_t i = 0;
    while ( i < r ) {
      size_t j = 0;
      while ( j < c ) {
	SingleEdit* se = new SingleEdit(sd.get_single(i,j));
	grid_.add(se,i,j);
	se->show();
	++j;
      }
      ++i;
    }
    add(&grid_);
    grid_.show();
    grid_added_ = true;
  }

  connect_to_method(sd.row_added,
		    this,
		    &SingleEditGroup::add_row);
  connect_to_method(sd.row_deleted,
		    this,
		    &SingleEditGroup::delete_row);
  connect_to_method(sd.column_added,
		    this,
		    &SingleEditGroup::add_column);
  connect_to_method(sd.column_deleted,
		    this,
		    &SingleEditGroup::delete_column);

}

SingleEditGroup::~SingleEditGroup()
{

}

void 
SingleEditGroup::install_grid()
{
  if (!grid_added_) {
    if ( (grid_.rows() > 0) && (grid_.columns() > 0) ) {
      no_plots_.hide();
      remove(&no_plots_);
      add(&grid_);
      grid_.show();
      grid_added_ = true;
    }
  }
}

void 
SingleEditGroup::install_noplots()
{
  if (grid_added_) {
    if (grid_.rows() == 0 || grid_.columns() == 0) {
      grid_.hide();
      remove(&grid_);
      add(&no_plots_);
      no_plots_.show();
      grid_added_ = false;
    }
  }
}

void
SingleEditGroup::add_row(size_t r)
{
  if (r != grid_.rows()) {
    g_warning("Current implementation only appends, this is going to bomb.");
  }

  grid_.add_row();

  install_grid();

  size_t i = 0;
  size_t c = grid_.columns();
  
  while ( i < c ) {
    SingleEdit* se  = new SingleEdit(sd_.get_single(r,i));
    grid_.add(se,r,i);
    g_print("add_row: Attached SingleEdit %p\n",se);
    se->show();
    ++i;
  }
}

void
SingleEditGroup::delete_row(size_t r)
{
  grid_.delete_row(r);

  install_noplots();
}

void
SingleEditGroup::add_column(size_t c)
{
  if (c != grid_.columns()) {
    g_warning("Current implementation only appends, this is going to bomb.");
  }

  grid_.add_column();

  install_grid();

  size_t i = 0;
  size_t r = grid_.rows();
  while ( i < r ) {
    SingleEdit* se  = new SingleEdit(sd_.get_single(i,c));
    grid_.add(se,i,c);
    g_print("add_column: Attached SingleEdit %p\n",se);
    se->show();
    ++i;
  }
}

void
SingleEditGroup::delete_column(size_t c)
{
  grid_.delete_column(c);
  
  install_noplots();
}

////////////////////////////////////////////////////////////////////

class ScatterEditImpl : public Gtk_Table {
public:
  ScatterEditImpl(ScatterDoc & sd, GuppiDoc & gd);
  ~ScatterEditImpl();

private:
  ScatterDoc & sd_;
  GuppiDoc & gd_;

  SingleEditGroup se_group_;

  AxisEditGroup x_;
  AxisEditGroup y_;
  
  Gnome_Pixmap bloat_;
};

ScatterEdit::ScatterEdit(ScatterDoc & sd, GuppiDoc & gd)
  : Gtk_Frame(),
    impl_(new ScatterEditImpl(sd,gd))
{
  set_shadow_type(GTK_SHADOW_OUT);
  add(impl_);
}

ScatterEdit::~ScatterEdit()
{
  impl_->hide();
  delete impl_;
}

ScatterEditImpl::ScatterEditImpl(ScatterDoc & sd, GuppiDoc & gd)
  : Gtk_Table(2,2,false),
    sd_(sd), gd_(gd),
    se_group_(sd),
    x_(sd.x_axes,gd,_("X Axes"),_("X Axis %d"),_("Add X Axis"),
       GTK_ORIENTATION_HORIZONTAL), 
    y_(sd.y_axes,gd,_("Y Axes"),_("Y Axis %d"),_("Add Y Axis"),
       GTK_ORIENTATION_VERTICAL),
    bloat_()
{ 
  attach(&se_group_,1,2,0,1);
  x_.border_width(GNOME_PAD);
  y_.border_width(GNOME_PAD);
  attach(&x_,1,2,1,2);
  attach(&y_,0,1,0,1);
  gchar* logo = gnome_pixmap_file("guppi.xpm");
  if (logo) {
    bloat_.load("guppi.xpm");
    g_free(logo);
  }
  attach(&bloat_,0,1,1,2);
  show_all();
}

ScatterEditImpl::~ScatterEditImpl()
{

}
