// -*- C++ -*-

/* 
 * scatterplot.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */


#include "scatterplot.h"

#include "grid.h"
#include "axistarget.h"
#include "redrawer.h"

// A single plot (one scatter) drawing target
class ScatterTarget : public Gtk_Plot {
public:
  ScatterTarget(const SingleScatter & ss);
  ~ScatterTarget();

private:
  const SingleScatter& ss_;

  Redrawer redrawer_;

  void change_marker(const SingleScatter & ss);
  void change_size(const AxisDoc& ad);
  void change_markercolor(const SingleScatter & ss);

  void redraw();

  void change_axis_bounds(gdouble xmin, gdouble xmax,
			  gdouble ymin, gdouble ymax);

  ScatterTarget();
  friend class Redrawer;
};

ScatterTarget::ScatterTarget(const SingleScatter& ss)
  : ss_(ss), redrawer_(*this)
{
  connect_to_method(ss_.marker_changed,
		    this,
		    &ScatterTarget::change_marker);

  const ColumnAxisDoc* x_axis = &ss_.get_x_axis();
  const ColumnAxisDoc* y_axis = &ss_.get_y_axis();

  connect_to_method(x_axis->size_changed,
		    this,
		    &ScatterTarget::change_size);

  connect_to_method(y_axis->size_changed,
		    this,
		    &ScatterTarget::change_size);

  connect_to_method(ss_.color_changed,
		    this,
		    &ScatterTarget::change_markercolor);

  connect_to_method(new_bounds_request,
		    this,
		    &ScatterTarget::change_axis_bounds);

}

ScatterTarget::~ScatterTarget()
{

}

void 
ScatterTarget::redraw()
{
  const ColumnAxisDoc* x_axis = &ss_.get_x_axis();
  const ColumnAxisDoc* y_axis = &ss_.get_y_axis();
   
  set_x_bounds(x_axis->get_start(), x_axis->get_stop());
  set_y_bounds(y_axis->get_start(), y_axis->get_stop());

  Gtk_Plot::redraw(); // sets drawing target and does background
  gc_save();
  // draw a little border around the plot
  gc()->set_foreground("black");
  draw_rectangle(false,0,0,Gdk_Draw::width()-1,Gdk_Draw::height()-1);

  gc()->set_foreground(ss_.get_color());

  const ColumnDoc* x_col = x_axis->get_column();
  const ColumnDoc* y_col = y_axis->get_column();

  if (x_col == 0) { draw_string(30,30,_("No X Column")); gc_restore(); return; }
  if (y_col == 0) { draw_string(30,30,_("No Y Column")); gc_restore(); return; }

  // Can only plot the smallest number of points on either axis
  size_t minsize = MIN(x_col->ds().size(), y_col->ds().size());

  Plot_Data xpd(x_col->ds().data(),
		minsize);
  
  Plot_Data ypd(y_col->ds().data(),
		minsize);

  draw_scatter_plot(xpd,
		    ypd,
		    ss_.get_marker());

  gc_restore();
}

void 
ScatterTarget::change_marker(const SingleScatter & ss)
{
  redraw();
}

void 
ScatterTarget::change_size(const AxisDoc& ad)
{
  // Could get this from two axes in immediate succession, 
  //  thus we use the queue to avoid double redraw.
  redrawer_.queue();
}

void 
ScatterTarget::change_markercolor(const SingleScatter & ss)
{
  redraw();
}

void 
ScatterTarget::change_axis_bounds(gdouble xmin, gdouble xmax,
				  gdouble ymin, gdouble ymax)
{
  ColumnAxisDoc* x_axis = &ss_.get_x_axis();
  ColumnAxisDoc* y_axis = &ss_.get_y_axis();  

  // Results in a signal which propagates to other plots
  //  using the same axis.

  // FIXME this works right for all the plots except this one, 
  //  which is redrawn twice, since it gets two signals.
  x_axis->set_bounds(xmin,xmax);
  y_axis->set_bounds(ymin,ymax);
  
#ifdef GNOME_ENABLE_DEBUG
  g_print("ScatterTarget::new_bounds_request_impl\n");
#endif
}


// The collection of scatterplots, and the axes
// This is similar enough to SingleEditGroup that they could 
//  probably share a lot of code.

class ScatterPlotImpl : public Gtk_Table {
public:
  ScatterPlotImpl(const ScatterDoc & sd, 
		  ScatterPlot & sp);
  ~ScatterPlotImpl();

  void make_targets();

private:
  const ScatterDoc& sd_;
  ScatterPlot & sp_;

  Gtk_Grid grid_;
  AxisTargetGroup x_axes_;
  AxisTargetGroup y_axes_;
  
  Gtk_Label no_plots_;
  bool grid_added_;

  Gnome_Pixmap bloat_;

  void add_x_axis(ColumnAxisDoc& cad, size_t i);
  void add_y_axis(ColumnAxisDoc& cad, size_t i);

  void delete_x_axis(ColumnAxisDoc& cad, size_t i);
  void delete_y_axis(ColumnAxisDoc& cad, size_t i);

  void add_row(size_t r);
  void delete_row(size_t r);
  void add_column(size_t c);
  void delete_column(size_t c);

  void install_grid();
  void install_noplots();

  void setup_target(ScatterTarget & st);

  void show_x_spacing();
  void show_y_spacing();
};

ScatterPlot::ScatterPlot(const ScatterDoc & sd) 
  : Plot(),
    impl_(new ScatterPlotImpl(sd,*this))
{
#if 0
  set_events(get_events() | 
	     GDK_POINTER_MOTION_HINT_MASK | 
	     GDK_BUTTON1_MOTION_MASK |
	     GDK_BUTTON_PRESS_MASK |
	     GDK_BUTTON_RELEASE_MASK);
#endif

  add(impl_);
  impl_->show_all();
}

ScatterPlot::~ScatterPlot()
{
  impl_->hide();
  delete impl_;
}

/* ////////////////////////
   The table is a 2x2; the cells are 
   [ y axes ]  [ plots ]
   [ pad    ]  [ x axes ]

   This won't really do. We need axes above and to the right 
   of the plot, and we need spacing between the plots so that the
   axis labels aren't clipped.
*/

ScatterPlotImpl::ScatterPlotImpl(const ScatterDoc & sd,
				 ScatterPlot& sp)
  : Gtk_Table(2,2,false), sd_(sd), sp_(sp),
    x_axes_(Axis_Base::SOUTH, sd_.x_axes), 
    y_axes_(Axis_Base::WEST, sd_.y_axes),
    no_plots_(_("No plots.")), grid_added_(false)
{
  show_x_spacing();
  show_y_spacing();

  connect_to_method(sd_.x_axes.spacing_changed,
		    this,
		    &ScatterPlotImpl::show_x_spacing);

  connect_to_method(sd_.y_axes.spacing_changed,
		    this,
		    &ScatterPlotImpl::show_y_spacing);

  size_t r = sd_.rows();
  size_t c = sd_.columns();

  grid_.reserve(r,c);
  
  if (r == 0 || c == 0) {
    attach(&no_plots_,1,2,0,1);
    no_plots_.show();
    grid_added_ = false;
  }
  else {
    size_t i = 0;
    while ( i < r ) {
      size_t j = 0;
      while ( j < c ) {
	ScatterTarget* st = new ScatterTarget(sd_.get_single(i,j));
	grid_.add(st,i,j);
	setup_target(*st);
	++j;
      }
      ++i;
    }
    attach(&grid_,1,2,0,1);
    grid_.show();
    grid_added_ = true;      
  }

  connect_to_method(sd_.row_added,
		    this,
		    &ScatterPlotImpl::add_row);
  connect_to_method(sd_.row_deleted,
		    this,
		    &ScatterPlotImpl::delete_row);
  connect_to_method(sd_.column_added,
		    this,
		    &ScatterPlotImpl::add_column);
  connect_to_method(sd_.column_deleted,
		    this,
		    &ScatterPlotImpl::delete_column);

  attach(&x_axes_,1,2,1,2);
  x_axes_.show();
  attach(&y_axes_,0,1,0,1);
  y_axes_.show();

  attach(&bloat_,0,1,1,2);
  gchar* logo = gnome_pixmap_file("guppi.xpm");
  if (logo) {
    bloat_.load("guppi.xpm");
    g_free(logo);
  }
  bloat_.show();
}

ScatterPlotImpl::~ScatterPlotImpl()
{
  
}

void
ScatterPlotImpl::show_x_spacing()
{
  grid_.set_col_spacings(sd_.x_axes.get_spacing());
}

void
ScatterPlotImpl::show_y_spacing()
{
  grid_.set_row_spacings(sd_.y_axes.get_spacing());
}

void
ScatterPlotImpl::add_row(size_t r)
{
  if (r != grid_.rows()) {
    g_warning("Current implementation only appends, this is going to bomb.");
  }

  grid_.add_row();

  install_grid();

  size_t i = 0;
  size_t c = grid_.columns();
  
  while ( i < c ) {
    ScatterTarget* st  = new ScatterTarget(sd_.get_single(r,i));
    grid_.add(st,r,i);
    setup_target(*st);
#ifdef GNOME_ENABLE_DEBUG
    g_print("add_row: Attached ScatterTarget %p\n",st);
#endif
    ++i;
  }
}

void
ScatterPlotImpl::delete_row(size_t r)
{
  grid_.delete_row(r);

  install_noplots();
}

void
ScatterPlotImpl::add_column(size_t c)
{
  if (c != grid_.columns()) {
    g_warning("Current implementation only appends, this is going to bomb.");
  }

  grid_.add_column();

  install_grid();

  size_t i = 0;
  size_t r = grid_.rows();
  while ( i < r ) {
    ScatterTarget* st  = new ScatterTarget(sd_.get_single(i,c));
    grid_.add(st,i,c);
    setup_target(*st);
#ifdef GNOME_ENABLE_DEBUG
    g_print("add_column: Attached ScatterTarget %p\n",st);
#endif
    ++i;
  }
}

void
ScatterPlotImpl::delete_column(size_t c)
{
  grid_.delete_column(c);
  
  install_noplots();
}

void 
ScatterPlotImpl::install_grid()
{
  if (!grid_added_) {
    if ( (grid_.rows() > 0) && (grid_.columns() > 0) ) {
      no_plots_.hide();
      remove(&no_plots_);
      attach(&grid_,1,2,0,1);
      grid_.show();
      grid_added_ = true;
    }
  }
}

void 
ScatterPlotImpl::install_noplots()
{
  if (grid_added_) {
    if (grid_.rows() == 0 || grid_.columns() == 0) {
      grid_.hide();
      remove(&grid_);
      attach(&no_plots_,1,2,0,1);
      no_plots_.show();
      grid_added_ = false;
    }
  }
}

void 
ScatterPlotImpl::setup_target(ScatterTarget & st)
{
  sp_.connect_gtkplot(st);
  // hack fixme
  st.set_usize(150,150);
  st.show();
}



