// -*- C++ -*-

/* 
 * textimport.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com> and
 * Havoc Pennington <hp@emccta.com>.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include "textimport.h"
#include "datatypemenu.h"
#include <limits.h>

class SeparatorMenu : public Gtk_VBox {
public:
  // arg is separator to edit
  SeparatorMenu(string & separator);
  ~SeparatorMenu();

  enum Separators {
    NEWLINE,
    SPACE,
    TAB,
    CARRIAGE,
    CARRIAGE_NEWLINE,
    OTHER,
    SEPARATORS_LAST
  };

  static const char* const Separator_Labels[] = {
    N_("Newline"),
    N_("Space"),
    N_("Tab"),
    N_("Carriage Return"),
    N_("Carriage Return - Newline"),
    N_("Other")
  };

  static const char* const Separator_Texts[] = {
    "\n",
    " ",
    "\t",
    "\r",
    "\r\n",
    ""
  };

private:
  string & separator_;

  Gtk_OptionMenu optionmenu_;
  Gtk_Menu menu_;
  Gtk_MenuItem* menuitems_[SEPARATORS_LAST];
  Gtk_Entry entry_;

  void mi_activated(Separators s);
  void entry_changed();
};

const char* const 
SeparatorMenu::Separator_Labels[];

const char* const 
SeparatorMenu::Separator_Texts[];

SeparatorMenu::SeparatorMenu(string & separator) 
  : Gtk_VBox(FALSE,GNOME_PAD_SMALL),
    separator_(separator),
    optionmenu_(),
    menu_(),
    entry_()
{
  optionmenu_.set_menu(&menu_);
  
  int i = 0;
  bool use_other = true;
  while ( i < SEPARATORS_LAST ) {
    menuitems_[i] = new Gtk_MenuItem(Separator_Labels[i]);
    connect_to_method(menuitems_[i]->activate,
		      this,
		      &SeparatorMenu::mi_activated,
		      (Separators)i);
    menu_.append(menuitems_[i]);
    if ( separator == Separator_Texts[i] ) {
      optionmenu_.set_history(i);
      mi_activated((Separators)i);
      use_other = false;
    }
    menuitems_[i]->show();
    ++i;
  }

  if (use_other) {
    optionmenu_.set_history((int)OTHER);
    entry_.set_text(separator_);
  }

  connect_to_method(entry_.changed,
		    this,
		    &SeparatorMenu::entry_changed);

  pack_start(&optionmenu_,TRUE,TRUE);
  pack_start(&entry_,TRUE,TRUE);

  optionmenu_.show();
  entry_.show();
}

SeparatorMenu::~SeparatorMenu()
{
  menu_.hide();
  optionmenu_.remove_menu();

  int i = 0;

  while ( i < SEPARATORS_LAST ) {
    menuitems_[i]->hide();
    delete menuitems_[i];
    ++i;
  }
}

void 
SeparatorMenu::mi_activated(Separators s)
{
  if ( s == OTHER ) {
    entry_.set_sensitive(true);
  }
  else {
    entry_.set_sensitive(false);
  }
  // triggers entry_changed
  entry_.set_text(Separator_Texts[s]);
}

void
SeparatorMenu::entry_changed()
{
  separator_ = entry_.get_text();
}

//////////////////////////////////////////////////////////////

class NumItems : public Gtk_HBox {
public:
  NumItems(const string & label);
  ~NumItems();

  Signal0 changed;

  guint get_value() const { return (guint) spin_.get_value_as_int(); }
  void  set_value(guint v) { adj_.set_value(v); }
  
private:
  Gtk_Label      label_;

  Gtk_Adjustment adj_;

  Gtk_SpinButton spin_;

  NumItems();
};

NumItems::NumItems(const string & label) 
  : Gtk_HBox(FALSE,GNOME_PAD_SMALL),
    label_(label),
    // since we're getting the value as an int, we need a limit of INT_MAX
    adj_(0.0,0.0,INT_MAX,1.0,1.0,1.0),
    spin_(&adj_)
{
  border_width(GNOME_PAD_SMALL);
  label_.set_alignment(0.0,0.5);

  pack_start(&label_,TRUE,TRUE);
  pack_end(&spin_,TRUE,TRUE);
  
  label_.show();
  spin_.show();

  connect_to_signal(spin_.changed,
		    changed);
}

NumItems::~NumItems() 
{

}

//////////////////////////////////////////////////////////////

class TextImportDialogImpl : public Gtk_VBox {
public:
  TextImportDialogImpl(const string & text, 
		       TextImportParams & params);
  ~TextImportDialogImpl();

private:
  const string & text_;
  TextImportParams & params_;

  Gtk_RadioButton columnar_;
  Gtk_RadioButton not_columnar_;

  Gtk_Frame       row_sep_frame_;
  Gtk_Frame       element_sep_frame_;
  Gtk_HBox        row_sep_box_;
  Gtk_HBox        element_sep_box_;

  Gtk_Label       row_sep_label_;
  Gtk_Label       element_sep_label_;

  SeparatorMenu  row_separators_;
  SeparatorMenu  element_separators_;

  NumItems    num_columns_;
  NumItems    num_elements_;

  Gtk_CheckButton no_valuelimit_;

  // FIXME need a DataTypeMenu for every 
  //  column being imported.
  Gtk_OptionMenu typeoptionmenu_;
  DataTypeMenu   typemenu_;

  void flip_columnar() { params_.columnar = !params_.columnar; }
  void set_num_columns() { params_.set_columns(num_columns_.get_value()); }
  void set_num_elements() { params_.num_elements = num_elements_.get_value(); }
  void set_datatype(ColumnDoc::DataType t) { 
    // fixme, need one for every column
    params_.set_type(0,t); 
  }
  void no_valuelimit_toggled() {
    params_.limit_elements = ! no_valuelimit_.get_state();
    num_elements_.set_sensitive(params_.limit_elements);
  }
};

TextImportDialog::TextImportDialog(const string & text, 
				   TextImportParams & params)
  : Gnome_Dialog(_("Importing text file"),
		 GNOME_STOCK_BUTTON_OK,
		 GNOME_STOCK_BUTTON_CANCEL,
		 0),
    impl_(new TextImportDialogImpl(text,params))
{
  vbox()->pack_start(impl_,TRUE,TRUE);
  impl_->show_all();
}

TextImportDialog::~TextImportDialog()
{
  impl_->hide();
  delete impl_;
}

TextImportDialogImpl::TextImportDialogImpl(const string & text, 
					   TextImportParams & params)
  : Gtk_VBox(FALSE,GNOME_PAD), 
    text_(text), params_(params),
    columnar_(0, _("Each row of text contains one value for each column")), 
    not_columnar_(columnar_.group(),_("Each row of text has all the values for a single column")),
    row_sep_frame_(),
    element_sep_frame_(),
    row_sep_box_(FALSE,GNOME_PAD_SMALL),
    element_sep_box_(FALSE,GNOME_PAD_SMALL),
    row_sep_label_(_("Rows of text are separated by: ")),
    element_sep_label_(_("Values within a row are separated by: ")),
    row_separators_(params.row_separator),
    element_separators_(params.element_separator),
    num_columns_(_("Maximum number of columns to import: ")),
    num_elements_(_("Maximum number of values per column: ")),
    no_valuelimit_(_("No limit on number of values")),
    typeoptionmenu_(),
    typemenu_()
{
  pack_start(&columnar_,TRUE,TRUE);
  pack_start(&not_columnar_,TRUE,TRUE);

  if (params_.columnar) {
    columnar_.set_state(true);
  } 
  else {
    not_columnar_.set_state(true);
  }

  // only have to connect to one, since they're grouped.
  connect_to_method(columnar_.toggled,
		    this,
		    &TextImportDialogImpl::flip_columnar);

  columnar_.show();
  not_columnar_.show();

  pack_start(&row_sep_frame_,TRUE,TRUE);
  pack_start(&element_sep_frame_,TRUE,TRUE);  

  row_sep_frame_.add(&row_sep_box_);
  element_sep_frame_.add(&element_sep_box_);

  row_sep_label_.set_alignment(0.0,0.5);
  element_sep_label_.set_alignment(0.0,0.5);
  row_sep_box_.pack_start(&row_sep_label_,TRUE,TRUE);
  element_sep_box_.pack_start(&element_sep_label_,TRUE,TRUE);
  
  row_sep_box_.pack_end(&row_separators_,TRUE,TRUE);
  element_sep_box_.pack_end(&element_separators_,TRUE,TRUE);

  row_sep_frame_.show_all();
  element_sep_frame_.show_all();

  pack_start(&num_columns_,TRUE,TRUE);
  pack_start(&num_elements_,TRUE,TRUE);

  num_columns_.set_value(params_.get_columns());
  num_elements_.set_value(params_.num_elements);
  num_elements_.set_sensitive(params_.limit_elements);

  connect_to_method(num_columns_.changed,
		    this,
		    &TextImportDialogImpl::set_num_columns);

  connect_to_method(num_elements_.changed,
		    this,
		    &TextImportDialogImpl::set_num_elements);

  num_columns_.show();
  num_elements_.show();

  pack_start(&no_valuelimit_,TRUE,TRUE);
  no_valuelimit_.set_state(!params_.limit_elements);
  no_valuelimit_.show();

  connect_to_method(no_valuelimit_.toggled,
		    this,
		    &TextImportDialogImpl::no_valuelimit_toggled);

  typeoptionmenu_.set_menu(&typemenu_);
  pack_start(&typeoptionmenu_,TRUE,TRUE);

  // the 0 is wrong of course; need to handle multiple columns.
  typeoptionmenu_.set_history((gint)params_.get_type(0));

  connect_to_method(typemenu_.datatype_activated,
		    this,
		    &TextImportDialogImpl::set_datatype);

  typeoptionmenu_.show();
}

TextImportDialogImpl::~TextImportDialogImpl()
{
  typeoptionmenu_.remove_menu();
}

